/***************************************************************************
 *   Copyright (C) 2004 by David Moreno Montero                            *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#ifndef __ALSAMIDI_H__
#define __ALSAMIDI_H__

#include <qobject.h>
#include <qptrlist.h>
#include <alsa/asoundlib.h>
#include <qthread.h>

#include "midievent.h"


struct pollfd;
class Track;
class Tracker;
class SysExBag;

/**
 * @short Helper container of port data
 */
class ALSAPort{
 public:
  /// Create a port of the selected ids and names
  ALSAPort(int _client, int _port, const QString &_c_name, const QString &_p_name){
    client=_client; port=_port;
    client_name=_c_name; port_name=_p_name;
  };
  /// Client ID
  int client;
  /// Port ID
  int port;
  /// Client name of this port
  QString client_name;
  /// Port name
  QString port_name;
};

/**
 * @short ALSA MIDI controller
 *
 * Opens a new ALSA MIDI client, with one input (current) and is able
 * to open new ports, each with the name of a track.
 *
 * The Track s send events when the Tracker marks time events.
 */
class Midi:public QObject{
  Q_OBJECT
public:
  Midi(SysExBag *bag, QObject *parent=0, const char *name="KMidiTracker");
  ~Midi();
  int addPort(const char *name);
  int addInputPort(const char *name);
  void updateMIDIInput(Tracker *);
  void updateMIDIInputBlock(Tracker *);
  void delPort(int id);
  void playNote(const MidiNote &ev, int MIDIChannel, int port);
  void playController(char controllerId, char val, int MIDIChannel, int port);
  void sendSysEx(const char *sysex, unsigned int len, int MIDIChannel, int port);
  void pitchBend(float value, int MIDIChannel, int port);
  void programChange(int instrument, int MIDIChannel, int port);
  void sendStartPlay();
  void sendStopPlay();
  void sendMIDIClock();
  void setPortName(int port, const char *name);


public slots:
  void recheckALSAPorts();
  void recheckALSAOutputPorts();
  void recheckALSAInputPorts();
  void resendPortList(); 
  void connectTrack(Track *, int client, int port);
  void setBlockMode(bool block);

  void setInputPort(int port);

signals:
 void ALSAOutputPortList(QPtrList<ALSAPort> &l);
 void ALSAInputPortList(QPtrList<ALSAPort> &l);

private:
  void decodeAlsaEvent(snd_seq_event_t *ev, Tracker *);


  /// Thread safe
  QMutex lock;
  /// Alsa sequencer handler
  snd_seq_t *handle;
  /// Id of the "current" port (input to current track)
  int currentPort;
  /// Is it in block mode
  bool inBlockMode;

  /// Last checked ALSA ports
  QPtrList<ALSAPort> inputLastPorts;
  /// Last checked ALSA ports
  QPtrList<ALSAPort> outputLastPorts;
  /// Registered ports
  QValueList<int> portList;
  /// SysEx bag.
  SysExBag *sysexbag;
};

 
#endif
