/***************************************************************************
 *   Copyright (C) 2004 by David Moreno Montero                            *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#ifndef __CONTROLLERCANVAS_H__
#define __CONTROLLERCANVAS_H__

#include <qcanvas.h>

class PatternView;
class ControllerText;
class Controller;
class VerticalText;

/**
 * @short View of a MIDI controller
 *
 * This widget is used by PatternView and with an associates Pattern
 * and an associated controller Id, it draws the controller. It is
 * also the controller, and changes the values when keyboard or mouse
 * select so.
 */
class ControllerCanvas : public QCanvasView{
  Q_OBJECT
public:
  ControllerCanvas(PatternView *patternView,int controller, QWidget *parent=0, const char *name=0);
  ~ControllerCanvas();

  void keyPressEvent ( QKeyEvent * e );
  void contentsMouseMoveEvent(QMouseEvent* e);
  void contentsMousePressEvent(QMouseEvent* e);
  void contentsMouseReleaseEvent(QMouseEvent* e);
  void viewportWheelEvent(QWheelEvent *e);
  void focusInEvent(QFocusEvent *e);
  void focusOutEvent(QFocusEvent *e);

  void focusReallyOut();

signals:
  void showControllerMenu();
  void changedController(unsigned int controllerId);

public slots:
  void redrawAll();
  void controllerChangedSlot(unsigned int controllerCol, unsigned int row, float val); 
  void controllerTextChangedSlot(unsigned int controller);
  void controllerChangedSlot(unsigned int row, float val);
  void controllerTextChangedSlot();
  void controllerChanged(unsigned int cn, Controller *c);
  void controllerChanged(Controller *c);
  void setPosition(int row);
  void setSignature(int a, int ){ beatLength=a; redrawAll(); }
  void showControllerChangeMenu();


protected:
  void drawAll();
  void deleteData();

  QColor getColor(bool dark=false);

  /// Number of rows
  int size;
  /// Controller id associated
  int controllerId;
  /// Height of each cell, got from the patternview
  int cheight;
  /// Last button pressed, usefull when dragging mouse
  int lastPressedButton;

  /// The beat length
  int beatLength;

  /// Actual keyposition (row position)
  int keypos;
  /// Background of each cell, to allow changing colors
  QCanvasRectangle *keyposmark;
  /// Each red mark
  QCanvasLine **mark;
  /// Each connection between marks (and one at begining and other at end)
  QCanvasLine **markConnection;
  /// The numbers of each cell
  QCanvasText **markNumber;
  /// The label that marks the controller id and name
  VerticalText *controllerNameLabel;
  /// Associated pattern view
  PatternView *patternView;

  /// The canvas where is drawn
  QCanvas canvas;

  /// The controller
  Controller *controller;
};

#endif
