/***************************************************************************
 *   Copyright (C) 2004,2005 by David Moreno Montero                       *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#include "controllersproperties.h"
#include "pattern.h"
#include "controller.h"
#include "midicontroller.h"
#include "sysexcontroller.h"
#include "sysex.h"
#include "sysexbag.h"
#include "pitchcontroller.h"

#include <qlabel.h>
#include <qbutton.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qlistbox.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qwidgetstack.h>
#include <kmessagebox.h>

ControllersProperties::ControllersProperties(Pattern *_pattern,
					     QWidget *parent, const char *name):
  QDialog(parent,name,FALSE,0){
  pattern=_pattern;

  setSizeGripEnabled(TRUE);

  QGroupBox *group;
  QHBox *hbox;
  QHBox *vbox;
  QPushButton *button;
  QHBox *grouphbox;
  QLabel *label;
  
  QVBoxLayout *layout=new QVBoxLayout(this,11,6);

  grouphbox=new QHBox(this);
  grouphbox->setSpacing(6);
  layout->addWidget(grouphbox);

  group=new QGroupBox("Current Controllers",grouphbox);
  group->setColumns(1);
  vbox=new QVBox(group);
  vbox->setSpacing(6);
  controllerList=new QListBox(vbox);
  connect(controllerList,SIGNAL(highlighted(int)),this,SLOT(selectedController(int)));
  hbox=new QHBox(vbox);
  hbox->setSpacing(6);
  button=new QPushButton("New",hbox);
  connect(button,SIGNAL(clicked()), this, SLOT(addControllerSlot()));
  button=new QPushButton("Delete",hbox);
  connect(button,SIGNAL(clicked()), this, SLOT(delControllerSlot()));
  layout->addWidget(group);

  group=new QGroupBox("Controller Properties",grouphbox);
  group->setColumns(1);
  vbox=new QVBox(group);
  vbox->setSpacing(6);
  hbox=new QHBox(vbox);
  hbox->setSpacing(6);
  new QLabel("Type: ",hbox);
  controllerTypeCombo=new QComboBox(hbox,"controller_type_combo");
  controllerTypeCombo->insertItem("MIDI Controller");
  controllerTypeCombo->insertItem("SysEx Controller");
  controllerTypeCombo->insertItem("Pitch Bend Controller");

  wstack=new QWidgetStack(vbox);

  button=new QPushButton("Change",vbox);
  connect(button,SIGNAL(clicked()), this,SLOT(changeController()));
  button=new QPushButton("Show SysEx Editor",vbox);
  connect(button,SIGNAL(clicked()),this,SIGNAL(showSysExEditor()));
  // Controller
  wstack->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::Maximum);
  controllerCombo=new QComboBox(wstack,"controller_combo");
  wstack->addWidget(controllerCombo,0);
  controllerCombo->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::Maximum);
  // SysEx
  vbox=new QVBox(wstack);
  vbox->setSpacing(6);
  wstack->addWidget(vbox,1);
  hbox=new QHBox(vbox);
  hbox->setSpacing(3);
  new QLabel("SysEx:",hbox);
  sysExCombo=new QComboBox(hbox,"sysex_combo");
  sysExCombo->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::Maximum);
  connect(sysExCombo,SIGNAL(highlighted(int)),this,SLOT(selectedSysEx(int)));
  new QLabel("Param:",hbox);
  sysExParamCombo=new QComboBox(hbox,"sysexparam_combo");
  sysExParamCombo->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::Maximum);
  
  SysExBag *sysexbag=pattern->getSysExBag();
  unsigned int i;
  for (i=0;i<127;i++)
    controllerCombo->insertItem(QString("%1. %2").
				arg(i).arg(MidiController::controllerName(i)));
  
  for (i=0;i<sysexbag->count();i++)
    sysExCombo->insertItem(sysexbag->at(i)->getName());

  connect(sysexbag, SIGNAL(newSysEx(const QString &)),
	  this, SLOT(newSysExController(const QString &)));
  connect(sysexbag, SIGNAL(newSysExParam(const QString &,const QString &)),
	  this, SLOT(newSysExParam(const QString &,const QString &)));
  connect(sysexbag, SIGNAL(delSysExParam(const QString &,int)),
	  this, SLOT(delSysExParam(const QString &,int)));
  connect(sysexbag, SIGNAL(delSysEx(const QString &)),
	  this, SLOT(delSysEx(const QString &)));

  // Pitch Bend
  label=new QLabel("",wstack);
  wstack->addWidget(label,2);

  // Final set up. Add controllers
  unsigned int n=pattern->getControllers();

  for (i=0;i<n;i++){
    Controller *c;
    c=pattern->getController(i);
    controllerList->insertItem(c->getText());
    connect(c, SIGNAL(deletedController(const QString &)),
	    this, SLOT(deletedController(const QString &)));
  }

  connect(controllerTypeCombo,SIGNAL(highlighted(int)),wstack,SLOT(raiseWidget(int)));


  label=new QLabel("Type of controller Unknown...\n How can these be?",wstack);
  wstack->addWidget(label,3);


  button=new QPushButton("Close",this);
  connect(button,SIGNAL(clicked()), this, SLOT(close()));
  layout->addWidget(button);
}

void ControllersProperties::addControllerSlot(){
  Controller *c=NULL;
  int type=controllerTypeCombo->currentItem();
  if (type==0){ // Midi controller
    c=new MidiController(controllerCombo->currentItem(), 
			 pattern->getMidi(), pattern->getRows());
    emit addController(c);
  }
  else if (type==1){
    SysExBag *sysexbag=pattern->getSysExBag();
    int nsysex=sysExCombo->currentItem();
    int nparam=sysExParamCombo->currentItem();
    if (nsysex>=sysExCombo->count() || nparam>=sysExParamCombo->count())
      return;
    c=new SysExController(sysexbag->at(nsysex),nparam,
			  pattern->getMidi(), pattern->getRows());
    emit addController(c);
  }
  else if (type==2){
    c=new PitchController(pattern->getMidi(), pattern->getRows());
    emit addController(c);
  }
}


void ControllersProperties::insertedController(int nc){
  Controller *c=pattern->getController(nc);
  connect(c, SIGNAL(deletedController(const QString &)),
	  this, SLOT(deletedController(const QString &)));
  controllerList->insertItem(c->getText());
  controllerList->setSelected(nc,true);
}

void ControllersProperties::selectedController(int nc){
  Controller *c=pattern->getController(nc);
  if (NULL==c)
    return;
  switch(c->controllerType()){
  case 1:
    controllerTypeCombo->setCurrentItem(0);
    wstack->raiseWidget(0);
    controllerCombo->setCurrentItem(((MidiController*)c)->getControllerId());
    break;
  case 2:
    controllerTypeCombo->setCurrentItem(1);
    wstack->raiseWidget(1);
    break;
  default:
    wstack->raiseWidget(2);
    break;
  }

}

void ControllersProperties::changeController(){
  Controller *c=pattern->getController(controllerList->currentItem());
  if (NULL==c)
    return;
  // Change of controller type
  if (c->controllerType()!=(controllerTypeCombo->currentItem()+1)){
    switch(controllerTypeCombo->currentItem()){
    case 0: // To midi controller
      pattern->changeController(controllerList->currentItem(),
				c=new MidiController(c,controllerCombo->currentItem()));
      controllerList->changeItem(c->getText(),controllerList->currentItem());
      break;
    case 1: // To SysEx controller
      {
	SysExBag *sysexbag=pattern->getSysExBag();
	int nsysex=sysExCombo->currentItem();
	int nparam=sysExParamCombo->currentItem();
	if (nsysex>=sysExCombo->count() || nparam>=sysExParamCombo->count())
	  return;
	pattern->changeController(controllerList->currentItem(),
				  c=new SysExController(c,sysexbag->at(nsysex),nparam));
	controllerList->changeItem(c->getText(),controllerList->currentItem());
      }
      break;
    case 2: // To Pitch Bend
      pattern->changeController(controllerList->currentItem(),
				c=new PitchController(c));
      controllerList->changeItem(c->getText(),controllerList->currentItem());
      break;
    default:
      break;
    }
  }
  // Change controller properties
  else{
    switch(c->controllerType()){
    case 1: // midi controller
      {
	MidiController *mc=(MidiController*)c;
	// Change of controller type
	if (mc->getControllerId()!=controllerCombo->currentItem()){
	  mc->setControllerId(controllerCombo->currentItem());
	  controllerList->changeItem(c->getText(),controllerList->currentItem());
	}
      }
      break;
    case 2: // sysex
      {
	SysExController *sc=(SysExController*)c;
	SysExBag *sysexbag=pattern->getSysExBag();
	int nsysex=sysExCombo->currentItem();
	int nparam=sysExParamCombo->currentItem();
	if (nsysex>=sysExCombo->count() || nparam>=sysExParamCombo->count())
	  return;
	sc->changeSysEx(sysexbag->at(nsysex),nparam);
	controllerList->changeItem(c->getText(),controllerList->currentItem());
      }
      break;
    default:
      printf("FIXME!\n");
    }
  }
}

/**
 * @short A controller type changed, so update interface
 *
 * The interface changes the controller list, and options avaliable if
 * it's the current selected controller
 */
void ControllersProperties::changedController(unsigned int controllerId){
  printf("CP: Changed controller %d\n",controllerId);

  Controller *c=pattern->getController(controllerId);
  connect(c, SIGNAL(deletedController(const QString &)),
	  this, SLOT(deletedController(const QString &)));

  controllerList->changeItem(pattern->getController(controllerId)->getText(), controllerId);
}

void ControllersProperties::delControllerSlot(){
  int ok;

  ok=KMessageBox::questionYesNo(this,QString("Are you sure you want to delete"
				" current selected controller\n%1").
				arg(controllerList->currentText()),"Sure to delete?");
  if (ok==KMessageBox::Yes){
    pattern->delController(controllerList->currentItem());
  }
}


void ControllersProperties::deletedController(const QString &name){
  unsigned int i;
  for (i=0;i<controllerList->count();i++)
    if (controllerList->text(i)==name)
      controllerList->removeItem(i);
}

void ControllersProperties::newSysExController(const QString &name){
  sysExCombo->insertItem(name);
  // Maybe it have params
  SysExBag *sysexbag=pattern->getSysExBag();
  SysEx *se=sysexbag->byName(name);
  int i;
  if (NULL!=se)
    for (i=0;i<se->numParams();i++)
      sysExParamCombo->insertItem(se->getName(i));
}

void ControllersProperties::newSysExParam(const QString &sysexname, const QString &paramname){
  if (sysexname==sysExCombo->currentText())
    sysExParamCombo->insertItem(paramname);
}

void ControllersProperties::delSysExParam(const QString &sysexname, int paramn){
  if (sysexname==sysExCombo->currentText()){
    sysExParamCombo->removeItem(paramn);
  }
}

void ControllersProperties::delSysEx(const QString &name){
  int i;
  for (i=0;i<sysExCombo->count();i++)
    if (sysExCombo->text(i)==name){
      sysExCombo->removeItem(i);
      return;
    }
}

void ControllersProperties::selectedSysEx(int nc){
  SysEx *sysex;
  int i;

  sysex=pattern->getSysExBag()->at(nc);
  if (NULL==sysex)
    return;
  
  sysExParamCombo->clear();
  for (i=0;i<sysex->numParams();i++)
    sysExParamCombo->insertItem(sysex->getName(i));
}

