/***************************************************************************
 *   Copyright (C) 2004 by David Moreno Montero                       *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/


#include <klocale.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <kkeydialog.h>
#include <kaccel.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kedittoolbar.h>
#include <kstdaccel.h>
#include <kaction.h>
#include <kstdaction.h>
#include <qcursor.h>
#include <kmessagebox.h>
#include <qtimer.h>
#include <kapplication.h>

#include "trackerbox.h"
#include "tracker.h"
#include "kmiditracker.h"
#include "trackview.h"
#include "pref.h"
#include "kmtfile.h"
#include "smffile.h"
#include "ksbfile.h"

/**
 * @short Creates a song instance
 *
 * It creates all the general dialogs (open, save...) and a basic
 * kmiditrackerView that holds all the real data.
 */
kmiditracker::kmiditracker()
    : KMainWindow( 0, "kmiditracker" )
{


    filename=QString::null;
    sysexname=QString::null;

    m_view=new kmiditrackerView(this);
    setCentralWidget(m_view);
    m_view->newTrack();

    // then, setup our actions
    setupActions();

    // and a status bar
    QStatusBar *status=statusBar();
    positionLabel=new QLabel("position",status);
    positionLabel->setFixedWidth(80);
    positionLabel->setFixedHeight(QFontMetrics(QFont()).height());
    status->addWidget(positionLabel,0,true);
    status->show();

    // apply the saved mainwindow settings, if any, and ask the mainwindow
    // to automatically save settings if changed: window size, toolbar
    // position, icon size, etc.
    setAutoSaveSettings();

    // allow the view to change the statusbar and caption
    connect(m_view, SIGNAL(changeStatusbar(const QString&,int)),
            this,   SLOT(changeStatusbar(const QString&,int)));
    connect(m_view, SIGNAL(positionChanged(unsigned char, unsigned char)),
            this,   SLOT(changePosition(unsigned char, unsigned char)));
    connect(m_view, SIGNAL(signalChangeCaption(const QString&)),
            this,   SLOT(changeCaption(const QString&)));
}

/**
 * @short Deletes the current view
 */
kmiditracker::~kmiditracker()
{
  delete m_view;
}

/**
 * @short load from a url
 */
void kmiditracker::load(const KURL& url)
{
}

/**
 * @short Initializes basic ations, as open, save...
 */
void kmiditracker::setupActions(){
  KStdAction::openNew(this, SLOT(fileNew()), actionCollection());
  KStdAction::open(this, SLOT(fileOpen()), actionCollection());
  KStdAction::save(this, SLOT(fileSave()), actionCollection());
  KStdAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
  
  new KAction(i18n("Save SysEx Bag"),"save_sysex",0,
	      this, SLOT(sysexSave()),actionCollection(),"save_sysex");
  new KAction(i18n("Save SysEx Bag As..."),"save_sysexas",0,
	      this, SLOT(sysexSaveAs()),actionCollection(),"save_sysexas");
  new KAction(i18n("Load SysEx Bag..."),"load_sysex",0,
	      this, SLOT(sysexOpen()),actionCollection(),"load_sysex");
  new KAction(i18n("Import SysEx Bag..."),"import_sysexas",0,
	      this, SLOT(sysexImport()),actionCollection(),"import_sysexas");

  
  
  KStdAction::quit(this, SLOT(askQuit()), actionCollection());

  m_toolbarAction = KStdAction::showToolbar(this, SLOT(optionsShowToolbar()), 
					    actionCollection());
  m_statusbarAction = KStdAction::showStatusbar(this, SLOT(optionsShowStatusbar()), 
						actionCollection());
  
  KStdAction::keyBindings(this, SLOT(optionsConfigureKeys()), actionCollection());
  KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());
  KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());
  
  connect(m_view, SIGNAL(showTrackMenu()), this, SLOT(showTrackMenu()));

  QTimer *timer=new QTimer(this);
  connect(timer, SIGNAL(timeout()), this, SLOT(changeHelpCookie()));
  timer->start(30000,FALSE);
  
  
  createGUI();
}



/**
 * @short Saves current properties
 *
 * FIXME
 */
void kmiditracker::saveProperties(KConfig *config)
{
    // the 'config' object points to the session managed
    // config file.  anything you write here will be available
    // later when this app is restored

}

/**
 * @short Reads properties from KConfig file
 */
void kmiditracker::readProperties(KConfig *config)
{
    // the 'config' object points to the session managed
    // config file.  this function is automatically called whenever
    // the app is being restored.  read in here whatever you wrote
    // in 'saveProperties'

    QString url = config->readPathEntry("lastURL");

    //    if (!url.isEmpty())
    //        m_view->openURL(KURL(url));
}

/**
 * @short Creates a new kmiditracker instance.
 */
void kmiditracker::fileNew()
{
    // this slot is called whenever the File->New menu is selected,
    // the New shortcut is pressed (usually CTRL+N) or the New toolbar
    // button is clicked

    // create a new window
    (new kmiditracker)->show();
}

/**
 * @short Open a file.
 *
 * It asks with a dialog the file to load, sets the filename QString,
 * and load() loads the data.  object.
 */
void kmiditracker::fileOpen(){
  QString s;
  s=KFileDialog::getOpenFileName(":<kmt>", i18n("*.kmt|KMidiTracker files (*.kmt)"),
				 this, i18n("Choose file name load from..."));
  
  if (s==QString::null){
    printf("No file choosed");
    return;
  }
  filename=s;
  load();
}

/**
 * @short Open a SysEx Bag file.
 *
 * It asks with a dialog the file to load, sets the filename QString,
 * and load() loads the data.  object.
 */
void kmiditracker::sysexOpen(){
  QString s;
  s=KFileDialog::getOpenFileName(":<ksb>", i18n("*.ksb|KMidiTracker SysEx Bag (*.ksb)"),
				 this, i18n("Choose file name to load from..."));
  
  if (s==QString::null){
    printf("No file choosed");
    return;
  }
  sysexname=s;

  KSBFile load(sysexname,m_view->getSysExBag());
  load.load();
}

/**
 * @short Open a SysEx Bag file.
 *
 * It asks with a dialog the file to load, sets the filename QString,
 * and load() loads the data.  object.
 */
void kmiditracker::sysexImport(){
  QString s;
  s=KFileDialog::getOpenFileName(":<ksb>", i18n("*.ksb|KMidiTracker SysEx Bag (*.ksb)"),
				 this, i18n("Choose file name to import from..."));
  
  if (s==QString::null){
    printf("No file choosed");
    return;
  }
  sysexname=s;

  KSBFile load(sysexname,m_view->getSysExBag());
  load.import();
}


/**
 * @short If no current name exists ask for a name fot the file, and
 * saves it
 */
void kmiditracker::fileSave(){
  if (filename==QString::null)
    fileSaveAs();
  else
    save();
}

/**
 * @short Always ask for a name to save to, and saves the data
 */
void kmiditracker::fileSaveAs(){
  QString s;
  s=KFileDialog::getSaveFileName(":<kmt>", 
				 i18n("*.kmt|KMidiTracker files (*.kmt)\n"
				      "*.mid|Standard MIDI files (*.mid)"),
				 this, i18n("Choose file name to save to..."));
  filename=QString::null;
  if (s==QString::null){
    printf("No file choosed");
    return;
  }
  QFile f(s); 
  if (f.exists()){
    int ok;
    ok=KMessageBox::questionYesNo(this,
				  QString(i18n("Are you sure you want"
					       " to overwrite\n%1?")).arg(s),
				  i18n("Sure to overwrite?"));
    if (ok!=KMessageBox::Yes)
      return;
  }

  filename=s;
  save();
}
/**
 * @short If no current name exists ask for a name fot the file, and
 * saves it. SysEx bag.
 */
void kmiditracker::sysexSave(){
  if (sysexname==QString::null){
    QString s;
    s=KFileDialog::getSaveFileName(":<ksb>", i18n("*.ksb|KMidiTracker SysEx Bag (*.ksb)"),
				   this, i18n("Choose file name to save to..."));

    if (s==QString::null){
      printf("No file choosed");
      return;
    }
    sysexname=s;
  }
  saveSysEx();
}

/**
 * @short Always ask for a name to save to, and saves the data
 */
void kmiditracker::sysexSaveAs(){
  QString s;
  s=KFileDialog::getSaveFileName(":<ksb>", i18n("*.ksb|KMidiTracker SysEx Bag (*.ksb)"),
				 this, i18n("Choose file name to save to..."));
  
  if (s==QString::null){
    printf("No file choosed");
    return;
  }
  sysexname=s;
  saveSysEx();
}

/**
 * @short Check if toolbar should be shown, and does
 */
void kmiditracker::optionsShowToolbar()
{
    // this is all very cut and paste code for showing/hiding the
    // toolbar
    if (m_toolbarAction->isChecked())
        toolBar()->show();
    else
        toolBar()->hide();
}

/**
 * @short Check if status bar should be shown, and does
 */
void kmiditracker::optionsShowStatusbar()
{
    // this is all very cut and paste code for showing/hiding the
    // statusbar
    if (m_statusbarAction->isChecked())
        statusBar()->show();
    else
        statusBar()->hide();
}

/**
 * @short If asked to configure keys, opens dialog to configure them
 */
void kmiditracker::optionsConfigureKeys(){
    KKeyDialog::configure(actionCollection(),this,true);
}

/**
 * @short Opens a new dialog about configuration of toolbars
 */
void kmiditracker::optionsConfigureToolbars(){
  saveMainWindowSettings( KGlobal::config(), "MainWindow" );
  KEditToolbar dlg(actionCollection());
  connect(&dlg,SIGNAL(newToolbarConfig()),this,SLOT(slotNewToolbarConfig()));
  if (dlg.exec())
    createGUI();
}

/**
 * @short Apply changes to the toolbar configuration
 */
void kmiditracker::newToolbarConfig(){
    // this slot is called when user clicks "Ok" or "Apply" in the toolbar editor.
    // recreate our GUI, and re-apply the settings (e.g. "text under icons", etc.)
    createGUI();

#if defined(KDE_MAKE_VERSION)
# if KDE_VERSION >= KDE_MAKE_VERSION(3,1,0)
    applyMainWindowSettings(KGlobal::config(), autoSaveGroup());
# else
    applyMainWindowSettings(KGlobal::config());
# endif
#else
    applyMainWindowSettings(KGlobal::config());
#endif
}

/**
 * @short Popups the preferences dialog
 *
 * FIXME
 */
void kmiditracker::optionsPreferences(){
    // popup some sort of preference dialog, here
    kmiditrackerPreferences dlg;
    if (dlg.exec())
    {
      emit reloadSettings();
    }
}

/**
 * @short display the text on the statusbar
 */
void kmiditracker::changeStatusbar(const QString& text,int ms){
  statusBar()->message(text,ms);
}

/**
 * @short Changes the position of the statusbar
 */ 
void kmiditracker::changePosition(unsigned char cpos, unsigned char rpos){
  positionLabel->setText(QString("<tt>Pos.%1:%2</tt>").arg(cpos+1,2).arg(rpos+1,2));
}

/**
 * @short display the text on the caption
 */
void kmiditracker::changeCaption(const QString& text){
    setCaption(text);
}

/**
 * @short Loads file filename
 */
void kmiditracker::load(){
  printf("Load to %s\n",filename.ascii());
  KMTFile f(filename, m_view);

  if (f.load()==-1)
    printf("Errororrororor");
}

/**
 * @short Saves data to filename
 */
void kmiditracker::save(){
  if (filename.right(4)==QString(".mid")){

    int ok;
    ok=KMessageBox::questionYesNo(this,
				  QString(i18n("Saving to Standard MIDI Format you loose "
					       "lots of information.\n\nAre you sure?")),
				  i18n("Sure to write to .mid?"));
    if (ok!=KMessageBox::Yes)
      return;
    
    printf("Saving to .mid\n");
    SMFFile f(filename, m_view);
    
    if (f.save()==-1)
      printf("Errororrororor");
  }
  else{
    KMTFile f(filename, m_view);
    
    if (f.save()==-1)
      printf("Errororrororor");
  }
}

/**
 * @short Saves sysex data to sysexname
 */
void kmiditracker::saveSysEx(){
  KSBFile f(sysexname, m_view->getSysExBag());

  if (f.save()==-1)
    printf("Errororrororor");
}

/**
 * @short Popups the track popup menu.xs
 */
void kmiditracker::showTrackMenu(){
  QWidget *w = factory()->container("track_popup", this);
  if (w==NULL)
    return;
  QPopupMenu *popup = static_cast<QPopupMenu *>(w);
  popup->exec(QCursor::pos());
}

void kmiditracker::closeEvent(QCloseEvent *e){
  int ok;
  ok=KMessageBox::questionYesNo(this,
				QString(i18n("Are you sure you want to quit?\n"
					     " All unsaved data will be lost.")),
				i18n("Sure to quit?"));
  if (ok==KMessageBox::Yes){
    e->accept();
    kapp->exit();
  }
}

void kmiditracker::askQuit(){
  kapp->closeAllWindows();
}



void kmiditracker::changeHelpCookie(){
  QString cookies[]={i18n("With F5, F6, F7 and F8 you can change focus to key areas"),
		     i18n("Double click on pattern name to change the name"),
		     i18n("With < and > change the notes velocity"),
		     i18n("All keybindings can be changed with Settings/Configure Shortcuts..."),
		     i18n("At the controller views, with Enter you enter/quit"
			  " the fine grain edition with cursors"),
		     i18n("With the middle button over key points on the controller view,"
			  " you remove the key point"),
		     i18n("There are several contextual menus with the second mouse button"),
		     i18n("You can define loop areas draging on the numbers at sequence views"),
		     i18n("Double click on pattern name to change the name"),
		     i18n("Backspace removes key points over controller view"),
		     i18n("Backspace inserts note off"),
		     i18n("Dot (.) removes a note"),
		     i18n("If you like KMidiTracker, send me a comment to:"
			  " david@monasteriomono.org to make me continue with the work"),
		     i18n("With Space you can start/stop the song")};

  int n;
  n=rand()%11;
  changeStatusbar(cookies[n],10000);
  printf("Help cookie: %s\n",cookies[n].ascii());
}
