/*
 * Copyright (C) 2003 David Moreno Montero <david@monasteriomono.org>
 */

#ifndef _KMIDITRACKER_H_
#define _KMIDITRACKER_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapplication.h>
#include <kmainwindow.h>

#include "kmiditrackerview.h"

class KToggleAction;
class TrackerBox;
class KAction;
class QLabel;

/**
 * @short Main window class
 *
 * This class serves as the main window for kmiditracker.  It handles the
 * menus, toolbars, and status bars.
 */
class kmiditracker : public KMainWindow
{
    Q_OBJECT
public:
    kmiditracker();
    virtual ~kmiditracker();

    void load(const KURL& url);
protected:
    void saveProperties(KConfig *);
    void readProperties(KConfig *);
    void closeEvent(QCloseEvent *e);

signals:
    void setViewToPattern();
    void setViewToPatternList();
    void setViewToSequence();

    void reloadSettings();

private slots:
    void fileNew();
    void fileOpen();
    void fileSave();
    void fileSaveAs();
    void sysexSave();
    void sysexSaveAs();
    void sysexOpen();
    void sysexImport();
    void optionsShowToolbar();
    void optionsShowStatusbar();
    void optionsConfigureKeys();
    void optionsConfigureToolbars();
    void optionsPreferences();
    void newToolbarConfig();

    void changePosition(unsigned char cpos, unsigned char rpos);
    void changeStatusbar(const QString& text,int ms);
    void changeCaption(const QString& text);

    void showTrackMenu();

    void askQuit();

    void changeHelpCookie();
private:
    void setupAccel();
    void setupActions();

    void save();
    void load();

    void saveSysEx();

private:
    /// Whether toolbar should be shown
    KToggleAction *m_toolbarAction;
    /// If statusbar should be shown
    KToggleAction *m_statusbarAction;

    /// Real view of kmiditracker
    kmiditrackerView *m_view;
    /// Actual filename, if any
    QString filename;
    /// Actual SysEx name
    QString sysexname;
    /// Current Position
    QLabel *positionLabel;
};

#endif // _KMIDITRACKER_H_
