/***************************************************************************
 *   Copyright (C) 2004 by David Moreno Montero                       *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#include "kmiditrackerview.h"
#include "trackview.h"
#include "sequenceview.h"
#include "trackerbox.h"
#include "tracker.h"
#include "alsamidi.h"
#include "sysexeditor.h"
#include "sysexbag.h"

#include <qlayout.h>
#include <qtabwidget.h>
#include <kmainwindow.h>
#include <kaction.h>
#include <kshortcut.h>
#include <kapplication.h>
#include <kconfig.h>

#include <klocale.h>


/**
 * @short Creates the view of the song
 *
 * The creation makes a initial Track, and creates lots of KAction s.
 */
kmiditrackerView::kmiditrackerView(KMainWindow *parent)
    : QWidget(parent,"kmiditrackerView"),
      DCOPObject("kmiditrackerIface")
{
  // Start the real Tracker
  trackerBox=new TrackerBox(&sysexbag,parent);

  connect(parent, SIGNAL(reloadSettings()),
	  this, SIGNAL(reloadSettings()));

  factory=parent->factory();
  trackerBox->hide();

  tracker=trackerBox->getTracker();

  connect(this, SIGNAL(reloadSettings()),
	  tracker, SLOT(reloadSettings()));
  connect(tracker, SIGNAL(midiNoteSignal(int,const MidiNote &)), 
	  this, SLOT(midiNote(int,const MidiNote &)));
	  

  //setup our layout manager to automatically add our widgets
  QVBoxLayout *top_layout = new QVBoxLayout(this);
  top_layout->setAutoAdd(true);
  
  view=new QTabWidget(this, "tracks_tab");
  //  view->setTabShape(QTabWidget::Triangular);

  sequence=new SequenceView(true,view,"Sequence View");
  sequence->setSignature(trackerBox->getTracker()->beatLength(),
			 trackerBox->getTracker()->beatsPerBar());
  view->addTab(sequence,i18n("Sequence View"));

  sysexEditor=new SysExEditor(&sysexbag,this);

  new KAction(i18n("Show SysEx Editor..."),"idea",0,
	      sysexEditor,SLOT(show()),parent->actionCollection(),"sysex_editor");

  connect(this, SIGNAL(newTrackAdded(Track*)), trackerBox,SLOT(addTrack(Track*)));
  connect(sequence, SIGNAL(newLoop(int,int)), tracker, SLOT(newLoop(int,int)));
  connect(sequence, SIGNAL(positionChanged(unsigned char, unsigned char)),
	  this, SIGNAL(positionChanged(unsigned char, unsigned char)));
  connect(tracker, SIGNAL(loopChanged(int,int)), sequence, SLOT(loopChanged(int,int)));
  connect(tracker, SIGNAL(newSignature(int,int)), sequence, SLOT(setSignature(int,int)));


  inputPortsAction=new KSelectAction(i18n("Default Midi Input"),KShortcut(),
				     this,SLOT(setInputPort()),
				     parent->actionCollection(),"midi_input");

  connect(tracker->getMidi(), SIGNAL(ALSAInputPortList(QPtrList<ALSAPort> &)), 
	  this, SLOT(ALSAInputPortList(QPtrList<ALSAPort> &)));


  new KAction(i18n("Add a new &Track"),"tab_new",0,
	      this, SLOT(newTrack()),parent->actionCollection(),"new_track");

  new KAction(i18n("Cycle Tab &Views"),"tab_cycle",0,
	      this, SLOT(cycleTabs()),parent->actionCollection(),"cycle_tabs");
  new KAction(i18n("&Reverse Cycle Tab Views"),"reverse_tab_cycle",0,
	      this, SLOT(reverseCycleTabs()),parent->actionCollection(),"reverse_cycle_tabs");

  KAction *a;
  /// Copy and paste
  a=KStdAction::copy(this, SLOT(doCopy()), parent->actionCollection());
  trackActions.append(a);
  a=KStdAction::paste(this, SLOT(doPaste()), parent->actionCollection());
  trackActions.append(a);

  /// Track actions

  a=new KAction(i18n("Set View to P&attern"),"view_text",0,
	      this, SLOT(setActivePatternView()),parent->actionCollection(),"pattern_view");
  trackActions.append(a);
  a=new KAction(i18n("Set View to Pattern &List"),"view_right",0,
	      this, SLOT(setActivePatternList()),parent->actionCollection(),"pattern_list");
  trackActions.append(a);
  a=new KAction(i18n("Set View to &Sequence"),"view_bottom",0,
	      this, SLOT(setActiveSequenceView()),parent->actionCollection(),"sequence_view");
  trackActions.append(a);


  a=new KAction(i18n("&Deletes current Track"),"tab_remove",0,
			     this, SLOT(delTrack()),parent->actionCollection(),"del_track");
  trackActions.append(a);
  a=new KAction(i18n("Add new pattern to current track"),"text_block",0,
		this, SLOT(addPattern()),parent->actionCollection(),"new_pattern");
  trackActions.append(a);
  a=new KAction(i18n("Copy pattern"),"editcopy",0,
		this, SLOT(copyPattern()),parent->actionCollection(),"copy_pattern");
  trackActions.append(a);
  a=new KAction(i18n("Delete last pattern from current track"),"editdelete",0,
		this, SLOT(addPattern()),parent->actionCollection(),"del_pattern");
  trackActions.append(a);
  a=new KAction(i18n("Add a new column to current pattern"),"indent",0,
		this, SLOT(addColumn()),parent->actionCollection(),"add_column");
  trackActions.append(a);
  a=new KAction(i18n("Delete last column from current pattern"),"unindent",0,
		this, SLOT(delColumn()),parent->actionCollection(),"del_column");
  trackActions.append(a);
  a=new KAction(i18n("Pattern Properties"),"configure_file",0,
		this, SLOT(showPatternProperties()),parent->actionCollection(),"pattern_properties");
  trackActions.append(a);
  a=new KAction(i18n("Controllers Properties"),"configure_file",0,
		this, SLOT(showControllersProperties()),parent->actionCollection(),"controllers_properties");
  trackActions.append(a);
  a=new KAction(i18n("Add a new controller to current pattern"),"view_right",0,
		this, SLOT(addController()),parent->actionCollection(),"add_controller");
  trackActions.append(a);
  a=new KAction(i18n("Delete last controller from current pattern"),"view_remove",0,
		this, SLOT(delController()),parent->actionCollection(),"del_controller");
  trackActions.append(a);

  muteToggle=new KToggleAction(i18n("Mute Track"),"kmix",0,
			       0, 0,parent->actionCollection(),"mute_track");
  a=muteToggle;
  connect(a,SIGNAL(toggled(bool)),this,SLOT(muteTrack(bool)));
  trackActions.append(a);

  midiThru=new KToggleAction(i18n("Midi &Thru"),"artsaudiomanager",0,
			     parent->actionCollection(),"midi_thru");
  
  followSong=new KToggleAction(i18n("Follow &Song"),"goto",0,
                             parent->actionCollection(),"follow_song");
  
  KConfig *config=kapp->config();
  config->setGroup("MIDI");
  midiThru->setChecked(config->readEntry("MidiThru","true")=="true");
  connect(midiThru, SIGNAL(toggled(bool)), this, SLOT(updateMidiThru(bool)));
  
  config->setGroup("General");
  followSong->setChecked(config->readEntry("FollowSong","false")=="true");
  connect(followSong, SIGNAL(toggled(bool)), this, SLOT(updateFollowSong(bool)));


  trackerBox->setNotPlaying();
}

/**
 * @short No work is done by the moment
 */
kmiditrackerView::~kmiditrackerView(){
  trackList.setAutoDelete(true);
  trackList.clear(); // i have to delete tracks before deleting tracker
  delete trackerBox;
}

/**
 * @short Updates the MIDI Thru status, and send the reload to tracks
 */
void kmiditrackerView::updateMidiThru(bool ){
  KConfig *config=kapp->config();
  
  config->setGroup("MIDI");
  config->writeEntry("MidiThru",midiThru->isChecked());
  config->sync();

  emit reloadSettings();
}

/**
 * @short Updates the follow song flag
 * 
 * When in follow song mode, the track view changes the pattern 
 * that is shown as the pattern changes inthe song when playing.
 */
void kmiditrackerView::updateFollowSong(bool){
  KConfig *config=kapp->config();

  config->setGroup("General");
  config->writeEntry("FollowSong",followSong->isChecked());
  config->sync();

  emit reloadSettings();
}


/**
 * @short Updates the menu to include all input ports avaliable
 *
 * The menu entry Tracker/Midi Input is updated with these values.
 */
void kmiditrackerView::ALSAInputPortList(QPtrList<ALSAPort> &l){
  ALSAPort *port;
  QStringList s;
  KConfig *config=kapp->config();
  int n=0,selected=-1;
  for (port=l.first();port;port=l.next(),n++){
    QString name=port->client_name+":"+port->port_name;
    s.append(name);
    if (name==config->readEntry("Input",QString::null)){
      tracker->getMidi()->setInputPort(n);
      selected=n;
    }
  }
  
  inputPortsAction->setItems(s);
  if (selected>=0)
    inputPortsAction->setCurrentItem(selected);
}


/**
 * @short Gets the current selected input port, and tells the midi module
 *
 * 
 */
void kmiditrackerView::setInputPort(){
  tracker->getMidi()->setInputPort(inputPortsAction->currentItem());

  KConfig *config=kapp->config();
  
  config->setGroup("MIDI");
  config->writeEntry("Input",inputPortsAction->currentText());
  config->sync();
}

/**
 * @short Changes the title 
 *
 * Used, by example, when a new track is selected.
 */
void kmiditrackerView::slotSetTitle(const QString& title){
    emit signalChangeCaption(title);
}

/**
 * @short Creates a new track for this view 
 *
 * When the track is created, addTrack(...) is called and this creates
 * the view.
 */
void kmiditrackerView::newTrack(){
  Track *tr;
  tr=new Track(&sysexbag, NULL, QString(i18n("Track%1")).arg(view->count()));
  tr->setSignature(trackerBox->getTracker()->beatLength(),
		   trackerBox->getTracker()->beatsPerBar());
  tr->addPattern();
  tr->setPatternInSequence(0,0);
  tr->setPatternInSequence(1,0xFFFF);

  addTrack(tr);
  view->setCurrentPage(view->count()-1);
}

/**
 * @short Adds a track to the view.
 *
 * The new track is appended correctly, and all signals are correcly
 * connected and emitted.
 */
void kmiditrackerView::addTrack(Track *tr){
  if (tr==NULL)
    return;

  trackview=new TrackView(tr, view, tr->name());
  view->addTab(trackview, trackview->name());
  sequence->manageTrack(trackview->getTrack());
  connect(view, SIGNAL(currentChanged(QWidget *)), this, SLOT(selectedChanged(QWidget*)));
  connect(trackview, SIGNAL(newLoop(int,int)), tracker, SLOT(newLoop(int,int)));
  connect(tracker, SIGNAL(loopChanged(int,int)), trackview, SIGNAL(loopChanged(int,int)));
  connect(tr, SIGNAL(nameChanged(Track*,const QString&)), 
	  this, SLOT(nameChanged(Track*,const QString&)));
  connect(tr, SIGNAL(deleteTrack(Track*)), this, SLOT(delTrack(Track *)));

  connect(tracker, SIGNAL(stopSignal()), tr, SLOT(stopNotes()));

  connect(tracker->getMidi(), SIGNAL(ALSAOutputPortList(QPtrList<ALSAPort> &)), 
	  trackview, SLOT(ALSAPortList(QPtrList<ALSAPort> &)));

  connect(trackview, SIGNAL(setALSAOutput(Track*,int,int)),
	  tracker->getMidi(), SLOT(connectTrack(Track*,int,int)));
  
  connect(trackview, SIGNAL(showTrackMenu()),
	  this, SIGNAL(showTrackMenu()));
  connect(tr, SIGNAL(muteChanged(Track *,bool)), this, SLOT(muteChanged(Track *,bool)));

  connect(trackview, SIGNAL(showSysExEditor()), sysexEditor, SLOT(show()));

  connect(trackview, SIGNAL(statusMessage(const QString&,int)), 
	  this, SIGNAL(changeStatusbar(const QString&,int)));

  connect(trackview, SIGNAL(positionChanged(unsigned char, unsigned char)), 
	  this, SIGNAL(positionChanged(unsigned char, unsigned char)));

  connect(this, SIGNAL(reloadSettings()),
	  trackview, SIGNAL(reloadSettingsSignal()));

  tracker->getMidi()->resendPortList();
  
  emit newTrackAdded(trackview->getTrack());

  trackList.append(tr);
}



/**
 * @short Removes the track from the lists
 *
 * The view is not deleted as it is autodeleted
 * (TrackView::trackDeleted(Track*))
 */
void kmiditrackerView::delTrack(Track *tr){
  int id;
  id=trackList.findRef(tr);
  if (id==-1)
    return;
  trackList.remove(tr);
}

/**
 * @short Deletes the track that the current view manages
 *
 * It deletes the Track, not the TrackView.
 */
void kmiditrackerView::delTrack(){
  if (trackview!=NULL){
    Track *tr=trackview->getTrack();
    tr->emitDeleteTrack(tr);
    delete tr;
  }
  //  trackview=NULL;
}

/**
 * @short The widget selected in the tabwidget changed
 *
 * So I set the title name to this, and if the new selected view is
 * not a TrackView, I disable the delete action.
 */
void kmiditrackerView::selectedChanged(QWidget *w){
  slotSetTitle(w->name());
  // if they are equal it's a TrackView, because there is a preceding
  // signal that changes the trackview to current selected if it's a
  // TrackView.
  if (w==sequence){
    trackview=NULL;
    KAction *i;
    for (i=trackActions.first();i;i=trackActions.next())
      i->setEnabled(false);
    muteToggle->setChecked(false);
  }
  else{
    KAction *i;
    for (i=trackActions.first();i;i=trackActions.next())
      i->setEnabled(true);
    trackview=(TrackView*)w;
    muteToggle->setChecked(trackview->getTrack()->isMutted());
  }
}


/**
 * @short Sets the focus to the PatternView
 */
void kmiditrackerView::setActivePatternView(){
  trackview->setActivePatternView();
}
/**
 * @short Sets the focus to the Pattern list
 */
void kmiditrackerView::setActivePatternList(){
  trackview->setActivePatternList();

}

/**
 * @short Sets the focus to the SequenceView
 */
void kmiditrackerView::setActiveSequenceView(){
  trackview->setActiveSequenceView();
}

/**
 * @short Cycle about tabs (sequenceview->track1->track2->...->sequenceview->...)
 */
void kmiditrackerView::cycleTabs(){
  view->setCurrentPage((view->currentPageIndex()+1)%(view->count()));
  changeStatusbar(QString("Activated new tab: %1").
		  arg(view->tabLabel(view->currentPage()).remove("&")),500);
}

/**
 * @short Cycle about tabs (sequenceview->track1->track2->...->sequenceview->...)
 */
void kmiditrackerView::reverseCycleTabs(){
  view->setCurrentPage((view->currentPageIndex()+view->count()-1)%(view->count()));
  changeStatusbar(QString("Activated new tab: %1").
		  arg(view->tabLabel(view->currentPage()).remove("&")),500);
}


/**
 * @short Returns a list of the tracks
 *
 * Very usefull when saving
 */
QPtrList<Track> kmiditrackerView::getTrackList(){
  return trackList;
}



/**
 * @short Sets the BPM
 */
void kmiditrackerView::setBPM(float bpm){
  tracker->setBPM(bpm);
}

/**
 * @short Clear all data of the tracks. Is mostly a delete.
 *
 * Used to load new songs, and set new files.
 */
void kmiditrackerView::clearTracks(){
  Track *i;
  trackList.setAutoDelete(true);
  trackList.clear();
  trackList.setAutoDelete(false);
}


/**
 * @short Changes the name of a desired track on the view
 *
 * The change is mainly in the tab.
 */
void kmiditrackerView::nameChanged(Track *tr,const QString &name){
  int id;
  QWidget *w;
  QString nname;

  id=trackList.findRef(tr);
  if (-1==id) return;

  if (tr->isMutted())
    nname=QString("(%1)").arg(name);
  else
    nname=name;

  w=view->page(id+1);
  view->changeTab(w,nname);
  w->setName(nname);

  if (trackview==w)
    slotSetTitle(w->name());
}

/**
void kmiditrackerView::midiNote(const MidiNote &ev){
  //  fd
}
 */

/**
 * @short Adds a Pattern to current track
 */
void kmiditrackerView::addPattern(){
  if (trackview)
    trackview->addPattern();
}

/**
 * @short Copies current Pattern in the current track
 */
void kmiditrackerView::copyPattern(){
  if (trackview)
    trackview->copyPattern();
}

/**
 * @short Deletes pattern in current track
 */
void kmiditrackerView::delPattern(){
  printf("Not yet!\n"); // FIXME
}

/**
 * @short Adds a column in the current Pattern in the current Track
 */
void kmiditrackerView::addColumn(){
  if (trackview)
    trackview->addColumn();
}

/**
 * @short Deletes last note column in the current Pattern in the current Track
 */
void kmiditrackerView::delColumn(){
  if (trackview)
    trackview->delColumn();
}

/**
 * @short Adds a controller to the current Pattern in the current Track
 */
void kmiditrackerView::addController(){
  if (trackview)
    trackview->addController();
}

/**
 * @short Deletes last controller in the current Pattern i the current Track
 */
void kmiditrackerView::delController(){
  if (trackview)
    trackview->delController();
}

/**
 * @short Show PatternProperties dialog of the current Pattern in the current Track
 */
void kmiditrackerView::showPatternProperties(){
  if (trackview)
    trackview->showPatternProperties();
}

/**
 * @short Show ControllersProperties dialog of the current Pattern in the current Track
 */
void kmiditrackerView::showControllersProperties(){
  if (trackview)
    trackview->showControllersProperties();
}

/**
 * @short Gets a midi note event, and redirect it to the correct trackview
 */
void kmiditrackerView::midiNote(int step,const MidiNote &ev){
  if (trackview)
    trackview->midiNote(step,ev);
}

/**
 * @short Mutes current track (or unmutes)
 */
void kmiditrackerView::muteTrack(bool mute){
  if (!trackview)
    return;
  Track *track=trackview->getTrack();
  if (!mute)
    track->unmute();
  else
    track->mute();
}

/**
 * @short As Muted changed in a Track
 */
void kmiditrackerView::muteChanged(Track *tr, bool mute){
  nameChanged(tr,tr->name());
  if (!trackview)
    return;
  if (trackview->getTrack()==tr)
    muteToggle->setChecked(mute);
}

/**
 * @short Copy the data at the current viewed pattern to the clipboard
 */
void kmiditrackerView::doCopy(){
  trackview->doCopy();
}

/**
 * @short Pastes current data at the clipboard to the current pattern, if possible
 */
void kmiditrackerView::doPaste(){
  trackview->doPaste();
}

