/***************************************************************************
 *   Copyright (C) 2004 by David Moreno Montero                       *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#ifndef _KMIDITRACKERVIEW_H_
#define _KMIDITRACKERVIEW_H_

#include <qwidget.h>
#include <kparts/part.h>
#include <kmiditrackeriface.h>
#include <kactionclasses.h>

#include "track.h"
#include "sysexbag.h"

class QPainter;
class KURL;
class Track;
class TrackView;
class QTabWidget;
class SequenceView;
class KMainWindow;
class Tracker;
class KAction;
class KToggleAction;
class SysExEditor;
class TrackerBox;

/**
 * This is the main view class for kmiditracker.  Most of the non-menu,
 * non-toolbar, and non-statusbar (e.g., non frame) GUI code should go
 * here.
 *
 * The internal component are several tabbed views of tracks, grouped by 
 * channel/midi_interface
 *
 * @short Main view
 * @author David Moreno Montero <david@monasteriomono.org>
 * @version 0.5.1
 */
class kmiditrackerView : public QWidget, public kmiditrackerIface
{
  Q_OBJECT

public:
  ///  Default constructor
  kmiditrackerView(KMainWindow *parent);
  /// Destructor
  virtual ~kmiditrackerView();

  QPtrList<Track> getTrackList();
  /// returns the tracker
  Tracker *getTracker(){ return tracker; };
  /// Returns the SysEx Bag
  SysExBag *getSysExBag(){ return &sysexbag; };

signals:
  /// Use this signal to change the content of the statusbar
  void changeStatusbar(const QString& text, int ms);
  /// Use this signal to change the position at the statusbar
  void positionChanged(unsigned char, unsigned char);
  /// Use this signal to change the content of the caption
  void signalChangeCaption(const QString& text);
  /// A new track is added
  void newTrackAdded(Track *);

  void newLoop(int begin,int end);
  void loopChanged(int begin, int end);

  void activateLoop(bool);
  void showTrackMenu();

  void reloadSettings();

public slots:
  void slotSetTitle(const QString& title);
  void newTrack(); 
  void delTrack(); 
  void selectedChanged(QWidget *);
  void setBPM(float);
  void muteTrack(bool);

  void setActivePatternView();
  void setActivePatternList();
  void setActiveSequenceView();
  void cycleTabs();
  void reverseCycleTabs();

  void addPattern();
  void copyPattern();
  void delPattern();
  void showPatternProperties();
  void showControllersProperties();
  void addColumn();
  void delColumn();
  void addController();
  void delController();

  void clearTracks();
  void addTrack(Track *track);
  void delTrack(Track *track);

  void nameChanged(Track *,const QString &);
  void muteChanged(Track *tr, bool mute);

  void midiNote(int step, const MidiNote &ev);

  void setInputPort();
  void ALSAInputPortList(QPtrList<ALSAPort> &l);

  void updateMidiThru(bool);
  void updateFollowSong(bool);

  void doCopy();
  void doPaste();
private:
  /// The main sequence view
  SequenceView *sequence;

  /// List of avaliable tracks.
  QPtrList<Track> trackList;
  /// Actions related to tracks. Shaded when not in a track view
  QPtrList<KAction> trackActions;
  /// List of input ports
  KSelectAction *inputPortsAction;
  /// Mute toggle action
  KToggleAction *muteToggle;
  /// Midi Thru
  KToggleAction *midiThru;
  /// Follow the song as playing
  KToggleAction *followSong;

  /// Tab widgets
  QTabWidget *view; 
  /// Actual track in view, if any (or NULL)
  TrackView *trackview;
  /// Associated tracker
  Tracker *tracker;
  /// Factory to work with KAction
  KXMLGUIFactory *factory;
  /// The SysEx Editor and container
  SysExEditor *sysexEditor;
  /// The SysEx Bag
  SysExBag sysexbag;
  /// The tracker controller box
  TrackerBox *trackerBox;
};

#endif // _KMIDITRACKERVIEW_H_
