/***************************************************************************
 *   Copyright (C) 2004,2005 by David Moreno Montero                       *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#include <qxml.h>
#include <qdom.h>
#include <qfile.h>

#include <iostream>
using namespace std;

#include "ksbfile.h"
#include "sysex.h"
#include "sysexbag.h"


KSBFile::KSBFile(const QString &name, SysExBag *bag){
  sysexbag=bag;
  filename=name;
  doc=new QDomDocument("KMidiTracker_SysExBag");
}

KSBFile::KSBFile(QDomDocument *d, SysExBag *bag){
  sysexbag=bag;
  filename=QString::null;
  doc=d;
}

KSBFile::~KSBFile(){
  if(filename!=QString::null)
    delete doc;
}

int KSBFile::save(){
  printf("save to %s\n",filename.ascii());

  QFile file(filename);
  if (!file.open(IO_WriteOnly))
    return -1;
  
  saveData();

  QTextStream t(&file);
  QString s=doc->toString();
  t<<s;
  printf("%s\n",s.ascii());

  file.close();

  return 0;
}

QDomElement KSBFile::saveData(const QString &tagname){
  QDomElement root=doc->createElement(tagname);
  root.setAttribute("version","0.5.8");
  doc->appendChild(root);
  QDomElement el;
  unsigned int i;
  for (i=0;i<sysexbag->count();i++){
    el=createSysEx(sysexbag->at(i));
    root.appendChild(el);
  }
  return root;
}


QDomElement KSBFile::createSysEx(SysEx *s){
  QDomElement ret=doc->createElement("sysex");
  QDomElement el;

  ret.setAttribute("name",s->getName());
  QString ds="";
  QByteArray sysexdata=s->getSysEx();
  unsigned int i;
  for (i=0;i<sysexdata.count();i++)
    ds+=QString("00%1 ").arg(int(sysexdata[i])&0x0FF,2,16).right(3).upper();
  el=doc->createElement("data");
  QDomText data=doc->createTextNode(ds);
  el.appendChild(data);
  ret.appendChild(el);

  for (i=0;i<unsigned(s->numParams());i++){
    el=doc->createElement("param");
    el.setAttribute("name",s->getName(i));
    el.setAttribute("position",s->paramPosition(i));
    el.setAttribute("length",s->paramLength(i));
    ret.appendChild(el);
  }

  return ret;
}

int KSBFile::load(){
  sysexbag->clear();
  return import();
}


int KSBFile::import(){
  printf("load from %s\n",filename.ascii());

  QFile file(filename);
  if (!file.open(IO_ReadOnly))
    return -1;
  if (!doc->setContent(&file)){
    file.close();
    return -1;
  }
  file.close();

  // Check is what I want.
  QDomElement root=doc->documentElement();

  if (root.tagName()!="kmtsysexbag" && root.tagName()!="sysexbag") 
    return -1;
  loadData(root);

  return 0;
}

void KSBFile::loadData(QDomElement &root){
  cout <<root.tagName()<<endl;

  QDomNode n=root.firstChild();
  while(!n.isNull()){
    QDomElement e = n.toElement();
    if(!e.isNull()){
      if (e.tagName()=="sysex")
      	sysexbag->append(readSysEx(e));
      else 
	cout <<"Unknown element"<<e.tagName()<<endl;
    }
    n=n.nextSibling();
  }
}

SysEx *KSBFile::readSysEx(const QDomElement &s){
  QDomNode n=s.firstChild();
  QString name;
  char emptySysEx[]={0xF0, 0xF7};
  name=s.attribute("name","Unknown");
  SysEx *ret=new SysEx(name,emptySysEx);;

  while (!n.isNull()){
    QDomElement e=n.toElement();
    if (!e.isNull()){
      if (e.tagName()=="data"){
	ret->setMessage(e.text());
      }
      else if (e.tagName()=="param"){
	int init, length;
	QString name;
	name=e.attribute("name","Unknown");
	init=e.attribute("position","1").toInt();
	length=e.attribute("length","1").toInt();
	ret->addParam(init,length,name);
      }
    }
    n=n.nextSibling();
  }

  cout <<"New sysex "<<name<<", "<<ret->length()<<endl;
  return ret;
}
