/***************************************************************************
 *   Copyright (C) 2004 by David Moreno Montero                            *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#include "kmiditracker.h"
#include <kapplication.h>
#include <dcopclient.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <klocale.h>

static const char description[] =
    I18N_NOOP("KDE MIDI tracker");

static const char version[] = "0.5.13";

static KCmdLineOptions options[] =
{
    { "+[URL]", I18N_NOOP( "Document to open." ), 0 },
    KCmdLineLastOption
};

/**
 * \mainpage KMidiTracker
 *
 * \section intro Introduction
 *
 * KMidiTracker is a simple MIDI Tracker for ALSA
 * (http://www.alsa-project.org) and KDE (http://www.kde.org).
 *
 * The Tracker is similar in concept to a Step Sequencer, where in
 * several fixed step positions you set the notes that should be
 * played. 
 *
 * The song data is arranged in Track s, and each Track have a
 * sequence of several Pattern s. Each pattern is composed of notes.
 *
 * The arrangment of the notes on the Pattern s are time is set in the
 * rows (first row is time-zero), and on the columns you can have
 * polyphony and/or MIDI controllers.
 * 
 */


/**
 * @short Parses input args and creates a KMidiTracker object 
 *
 * 
 */
int main(int argc, char **argv)
{
    KAboutData about("kmiditracker", I18N_NOOP("kmiditracker"), version, description,
                     KAboutData::License_GPL, "(C) 2004 David Moreno Montero", 0, 0, "david@monasteriomono.org");
    about.addAuthor( "David Moreno Montero", 0, "david@monasteriomono.org" );
    KCmdLineArgs::init(argc, argv, &about);
    KCmdLineArgs::addCmdLineOptions(options);
    KApplication app;

    // register ourselves as a dcop client
    app.dcopClient()->registerAs(app.name(), false);

    // see if we are starting with session management
    if (app.isRestored())
    {
        RESTORE(kmiditracker);
    }
    else
    {
        // no session.. just start up normally
        KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
        if (args->count() == 0)
        {
            kmiditracker *widget = new kmiditracker;
            widget->show();
        }
        else
        {
            int i = 0;
            for (; i < args->count(); i++)
            {
                kmiditracker *widget = new kmiditracker;
                widget->show();
                widget->load(args->url(i));
            }
        }
        args->clear();
    }

    return app.exec();
}
