/***************************************************************************
 *   Copyright (C) 2004,2005 by David Moreno Montero                       *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#include "midiout.h"
#include "midicontroller.h"
#include "alsamidi.h"

/**
 * @short Creates the midi controller.
 */
MidiController::MidiController(int _controllerId, 
			       MidiOut *out, unsigned int size):
  Controller(out,size){
  controllerId=_controllerId;
  lastvalue=-2.0;
}

/**
 * @short Creates the midi controller.
 */
MidiController::MidiController(Controller *c, 
			       int _controllerId):
  Controller(c){
  controllerId=_controllerId;
  lastvalue=-2.0;
  minimum=0;
  maximum=127;
  bigstep=16;
}

/**
 * @short Send midi message if appropiate.
 *
 * FIXME
 */
void MidiController::step(int step){
  if (step>=1 && step<=signed(size)){
    unsigned char value=int(values[step-1]*63.5)+64;
    if (value!=lastvalue || step==1){
      midi->playController(controllerId, value);
      lastvalue=value;
    }
  }
}

/**
 * @short Sets the controller text. 
 *
 * It is the controller number and a special text.
 */
QString MidiController::getText(){
  return QString("Controller %1: %2").
    arg(controllerId).arg(controllerName(controllerId));
}

/**
 * @short Selects the correct name for a controller id, if any
 * avaliable.
 *
 * If no known name exists, returns Unknown.
 */
QString MidiController::controllerName(int n){
  struct{
    char id;
    const char *name;
  } names[]={
    {0, "Bank Select"},
    {1, "Modulation Wheel"},
    {2, "Breath Controller"},
    {4, "Foot Controller"},
    {5, "Portamento Time"},
    {6, "Data Entry"},
    {7, "Volume"},
    {8, "Balance"},
    {10, "Pan"},
    {11, "Expression"},
    {12, "Effect Control 1"},
    {13, "Effect Control 2"},
    {16, "General Purpose #1"},
    {17, "General Purpose #2"},
    {18, "General Purpose #3"},
    {19, "General Purpose #4"},
    {64, "Hold 1 (Sustain)"},
    {65, "Portamento On/Off"},
    {66, "Sostenuto"},
    {67, "Soft Pedal"},
    {68, "Legato Footswitch"},
    {69, "Hold 2"},
    {70, "Sound Controller 1 / Sound Variation"},
    {71, "Sound Controller 2 / Timbre Content"},
    {72, "Sound Controller 3 / Release Time"},
    {73, "Sound Controller 4 / Attack Time"},
    {74, "Sound Controller 5 / Brightness"},
    {75, "Sound Controller 6"},
    {76, "Sound Controller 7"},
    {77, "Sound Controller 8"},
    {78, "Sound Controller 9"},
    {79, "Sound Controller 10"},
    {80, "General Purpose #5"},
    {81, "General Purpose #6"},
    {82, "General Purpose #7"},
    {83, "General Purpose #8"},
    {84, "Portamento Control"},
    {91, "Effect 1 / External Effects Depth"},
    {92, "Effect 2 / Tremelo Depth"},
    {93, "Effect 3 / Chorus Depth"},
    {94, "Effect 4 / Detune (Celeste)"},
    {95, "Effect 5 / Phaser Depth"},
    {96, "Data Increment"},
    {97, "Data Decrement"},
    {98, "Non-Registered Parameter lsb"},
    {99, "Non-Registered Parameter msb"},
    {100, "Registered Parameter lsb"},
    {101, "Registered Parameter msb"},
    {-1, NULL}
  };

  int i=0;
  while (names[i].id!=-1){
    if (names[i].id==n)
      return QString(names[i].name);
    i++;
  }
  return QString("Unknown");
}
