/***************************************************************************
 *   Copyright (C) 2004 by David Moreno Montero                            *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#ifndef __MIDIEVENT_H__
#define __MIDIEVENT_H__

#include <qevent.h>


/**
 * @short MidiNotes data. 
 *
 * No real actions are done in the class.
 */
class MidiNote{
protected:
  unsigned char _note;
  unsigned char _velocity;
public:
  enum extraNotes {NoNote=0xFE, StopNote=0xFF};

  inline MidiNote(unsigned int Note=NoNote, unsigned int Velocity=0){
    _note=Note; _velocity=Velocity&0x07F;
  }

  inline unsigned int note() const { return _note; }
  inline unsigned int velocity() const { return _velocity; }
};

/**
 * @short MidiNote QEvent cousin
 *
 * This is just the same as MidiNote but is also a QEvent, so you can
 * launch events and let the GUI thread precess it.
 */
class MidiNoteEvent:public QCustomEvent,public MidiNote{
public:
  MidiNoteEvent(unsigned int Note=NoNote, unsigned int Velocity=0):QCustomEvent(QEvent::User+1),MidiNote(Note,Velocity){};
  MidiNoteEvent(const MidiNote &note):QCustomEvent(QEvent::User+1),MidiNote(note){};
};

#endif
