/***************************************************************************
 *   Copyright (C) 2005 by David Moreno Montero                            *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#ifndef __MIDI_OUT_H__
#define __MIDI_OUT_H__

#include "alsamidi.h"

class MidiNote;

/**
 * @short Encapsulates information about a simple midi output
 *
 * A MIDI output is seen as the combination of the alsa midi out port,
 * the port and the channel.
 *
 * Tis class just accepts the same messages as the Midi object, but
 * help as there is no need to remember all data as in Midi class,
 * just a pointer to here, and then send the message.
 */
class MidiOut{
 protected:
  Midi *midi;
  char channel;
  int port;

 public:
  MidiOut(){ midi=NULL; }

  MidiOut(Midi *_midi, int _channel, int _port){
    midi=_midi;
    channel=_channel;
    port=_port;
  };

  MidiOut(const MidiOut &m){
    midi=m.midi;
    channel=m.channel;
    port=m.port;
  };

  char getChannel() const { return channel; };
  bool isValid() const { return midi!=NULL; };

  void setMidi(Midi *_midi){ midi=_midi; };
  void setChannel(int _channel){ channel=_channel; };
  void setPort(int _port){ port=_port; };

  void playNote(const MidiNote &ev){ 
    if (midi!=NULL) midi->playNote(ev,channel,port); }
  void playController(char controllerId, char val){ 
    if (midi!=NULL) midi->playController(controllerId,val,channel,port); }
  void sendSysEx(const char *sysex, unsigned int len){ 
    if (midi!=NULL) midi->sendSysEx(sysex,len,channel,port); }
  void pitchBend(float value){
    if (midi!=NULL) midi->pitchBend(value,channel,port); }
  

};


#endif
