/***************************************************************************
 *   Copyright (C) 2004 by David Moreno Montero                            *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#ifndef __PATTERNVIEW_H__
#define __PATTERNVIEW_H__

#include <qscrollview.h>
#include <qptrlist.h> 
#include "pattern.h"


class QHBox;
class PatternProp;
class ControllersProperties;
class QLabel;
class TrackView;
class ControllerCanvas;
class VerticalLabel;
class QPushButton;

/**
 * @short The view of all the pattern data
 *
 * The real data is in the Pattern object, and this class only does
 * the Qt/KDE view of all this data, using ControllerCanvas and lots
 * of QLabel.
 */
class PatternView : public QScrollView{
  Q_OBJECT
public:
  PatternView(Pattern *_pattern, TrackView *trackview, QWidget *parent=0, const char *name=0);
  ~PatternView();

  static QString noteText(const MidiNote &note);
  QString getPatternName();
  /// The pattern associated
  Pattern *getPattern(){ return pattern; }
protected:
  /// The real pattern
  Pattern *pattern;
  /// The columns
  QHBox *columns;
  /// The property box of this pattern. Each pattern have one. FIXME
  /// to be more efficient.
  PatternProp *patternProp;
  /// The controllers properties. Similar to PatternView::patternProp
  ControllersProperties *controllersProp;
  /// The note labels
  QLabel ***noteview;
  /// The parent trackView
  TrackView *trackView;
  /// The numbers on the left. We need pointers to change colors when playing.
  QPtrList<QLabel> numberList;
  /// Last number painted, to allow easy jump of position and change of colors
  QLabel *lastStepNumber;
  /// The controller views. Uses ControllerCanvas class.
  QPtrList<ControllerCanvas> controller;
  /// Pattern name label
  VerticalLabel *patternName;
  /// List of mute buttons
  QPtrList<QPushButton> muteButtons;
  
  /// Position of the pointer
  unsigned int cpos, rpos;
  /// Size of the pattern
  unsigned int rows, cols;
  /// the identifier of each pattern. Unique.
  int patternId;
  /// Base octave, as selected in the spin box.
  int baseOctave;
  /// The beat length
  int beatLength;

  /// Whether to do midi thru or not
  bool midiThru;
  /// Set focus always to the pointer?
  bool followSong;

  /// Selection bounds
  QRect selection;
  
  void deleteView();
  void constructView(void);
  void keyPressEvent(QKeyEvent *ev);
  void focusInEvent(QFocusEvent *);
  void focusOutEvent(QFocusEvent *);
  virtual void contentsMousePressEvent(QMouseEvent * e);
  virtual void contentsMouseMoveEvent(QMouseEvent * e);
  //virtual void contentsMouseReleaseEvent(QMouseEvent * e);
public:
  int getBaseOctave(){ return baseOctave; }
  int height();
  int cellHeight();
  int getBeatLength(){ return beatLength; }

signals:
  /// The pattern name is now name
  void setPatternName(int id, const QString &name);
  /// The number of rows changed
  void setRowsSignal(unsigned int rows);
  /// ??? FIXME
  void showTrackMenu();
  /// Sets the current position
  void positionChanged(unsigned char, unsigned char);

public slots:
  void askNewName();

  void changeNote(unsigned int row, unsigned int col, const MidiNote &note);

  void reconstructView(void);
  void setPosition(unsigned int row, unsigned int col);
  void setRowPos(unsigned int rpos);

  void addColumn();
  void delColumn();
  void addController();
  void addController(Controller *);
  void delController();
  void delController(int controller);
  void delController(const QString &name);
  void deletedController(const QString &name);

  void setRows(unsigned int nrows);
  void setCols(unsigned int ncols);
  void setName(const QString &name);
  void setPatternId(int id);
  void setBaseOctave(int octave);
  void clearStepPosition();
  void drawStepPosition();

  void focusInController(int n, int row);

  void showProperties(void);
  void showControllersProperties(void);
  
  void addMidiNote(int step, const MidiNote &newNote);

  void setSignature(int a,int ){ beatLength=a; reconstructView(); }

  void recheckMuteStatus();

  void reloadSettings();
  
  void newSelection(int,int,int,int);
  void removeSelection();
  
  void doCopy();
  void doPaste();
};

#endif
