/*
 * Copyright (C) 2004 David Moreno Montero <david@monasteriomono.org>
 */

#include "pref.h"

#include <klocale.h>
#include <kconfigskeleton.h>
#include <kapplication.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qslider.h>
#include <qcheckbox.h>

kmiditrackerPreferences::kmiditrackerPreferences()
  : KConfigDialog(NULL, "kmiditracker Preferences",new KConfigSkeleton(),
		  IconList, Ok|Apply|Cancel)
{
    // this is the base class for your preferences dialog.  it is now
    // a Treelist dialog.. but there are a number of other
    // possibilities (including Tab, Swallow, and just Plain)
    m_pageOne = new kmiditrackerPrefPageOne(this);
    addPage(m_pageOne, i18n("Midi"), "kcmmidi");
    
    m_pageTwo = new kmiditrackerPrefPageTwo(this);
    addPage(m_pageTwo, i18n("Performance"), "up");

    // Some config changed, change button status
    connect(m_pageOne, SIGNAL(configChanged()),
	    this, SLOT(settingsChangedSlot()));
    connect(m_pageTwo, SIGNAL(configChanged()),
	    this, SLOT(settingsChangedSlot()));

    connect(m_pageOne, SIGNAL(configChanged()),
	    this, SLOT(updateButtons()));
    connect(m_pageTwo, SIGNAL(configChanged()),
	    this, SLOT(updateButtons()));


    // Ok clicked, save status
    connect(this, SIGNAL(okClicked()),
	    m_pageOne, SLOT(saveChanges()));
    connect(this, SIGNAL(applyClicked()),
	    m_pageOne, SLOT(saveChanges()));
    connect(this, SIGNAL(okClicked()),
	    m_pageTwo, SLOT(saveChanges()));
    connect(this, SIGNAL(applyClicked()),
	    m_pageTwo, SLOT(saveChanges()));

    enableButtonOK(false);
    enableButtonApply(true);
}

void kmiditrackerPreferences::settingsChangedSlot(){
  enableButtonOK(true);
  enableButtonApply(true);
  //enableButton(Apply,true);
}

kmiditrackerPrefPageOne::kmiditrackerPrefPageOne(QWidget *parent)
    : QFrame(parent,"pref_pageone")
{
    QVBoxLayout *layout = new QVBoxLayout(this);
    layout->setAutoAdd(true);

    QHBox *hbox;
    QLabel *lab;
    KConfig *config=kapp->config();
    config->setGroup("MIDI");

    new QLabel(i18n("Midi Time Clock:"),this);
    hbox=new QHBox(this);
    new QLabel(i18n("MTC offset: "),hbox);
    MTCOffset=new QSlider(-32,32,1, 
			  config->readEntry("MTCOffset","0").toInt(), 
			  Qt::Horizontal, hbox);
    MTCLabel=new QLabel(QString("   %1").arg(config->readEntry("MTCOffset","0").toInt()),
			hbox);

    connect(MTCOffset, SIGNAL(valueChanged(int)), 
	    this, SLOT(MTCOffsetChanged(int)));

    SendMidiClock=new QCheckBox(i18n("Send Midi Time Clock messages"),this);
    SendMidiClock->setChecked(config->readEntry("SendMidiClock","true")=="true");
    new QLabel(i18n("Behaviour:"),this);
    MidiThru=new QCheckBox(i18n("Midi Thru"),this);
    MidiThru->setChecked(config->readEntry("MidiThru","true")=="true");

    connect(MidiThru, SIGNAL(clicked()), 
	    this, SIGNAL(configChanged()));
    connect(SendMidiClock, SIGNAL(clicked()), 
	    this, SIGNAL(configChanged()));
}

void kmiditrackerPrefPageOne::MTCOffsetChanged(int n){
  MTCLabel->setText(QString("  %1").arg(n));
  
  emit configChanged();
}

void kmiditrackerPrefPageOne::saveChanges(){
  KConfig *config=kapp->config();
  
  config->setGroup("MIDI");
  config->writeEntry("MTCOffset",MTCOffset->value());
  config->writeEntry("MidiThru",MidiThru->isChecked());
  config->writeEntry("SendMidiClock",SendMidiClock->isChecked());
  config->sync();
}

kmiditrackerPrefPageTwo::kmiditrackerPrefPageTwo(QWidget *parent)
    : QFrame(parent,"pref_pagetwo")
{
  KConfig *config=kapp->config();
  config->setGroup("Performance");
  QVBoxLayout *layout = new QVBoxLayout(this);
  layout->setAutoAdd(true);
  
  new QLabel(i18n("These settings are not avaliable until application restart"),this);

  rt=new QCheckBox(i18n("Try to use realtime scheduling"),this);
  rt->setChecked(config->readEntry("RealTime","true")=="true");

  hp=new QCheckBox(i18n("Try to use high priority"),this);
  hp->setChecked(config->readEntry("HighPriority","true")=="true");

  rtc=new QCheckBox(i18n("Try to use the Real Time Clock (RTC)"),this);
  rtc->setChecked(config->readEntry("RTC","true")=="true");


  connect(rt, SIGNAL(clicked()), this, SIGNAL(configChanged()));
  connect(hp, SIGNAL(clicked()), this, SIGNAL(configChanged()));
  connect(rtc, SIGNAL(clicked()), this, SIGNAL(configChanged()));
}

void kmiditrackerPrefPageTwo::saveChanges(){
  KConfig *config=kapp->config();
  config->setGroup("Performance");

  config->writeEntry("HighPriority",hp->isChecked());
  config->writeEntry("RealTime",rt->isChecked());
  config->writeEntry("RTC",rtc->isChecked());
  config->sync();
}

