/***************************************************************************
 *   Copyright (C) 2004 by David Moreno Montero                            *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#ifndef __SEQUENCEVIEW_H__
#define __SEQUENCEVIEW_H__

#include <qscrollview.h>
#include <qptrlist.h>
#include <qptrlist.h>
#include <qvbox.h>
#include <qlabel.h>

#include "track.h"

class QHBox;

/**
 * @short Simple view of the sequence arrangment of patterns
 *
 * With this class you have the view (of the MVC) of the sequence that
 * lies inside the Track object.
 *
 * This view can manage several tracks if you add them, or only
 * one. You may have as many sequence views as you want.
 */
class SequenceView:public QScrollView{
  Q_OBJECT
public:
  SequenceView(bool drawTitles, QWidget *parent, const char *name=0);
  ~SequenceView();

protected:
  /// The tracks this view manages
  QPtrList<Track> trackList;
  /// The main widget arrangment
  QHBox *hbox;
  /// Each off the columns of the view, including numbers
  QPtrList<QVBox> column;
  /// Labels of the sequences, 
  QPtrList<QPtrList<QLabel> > label;
  /// The list of the number list (first column)
  QPtrList<QLabel> numberList;
  /// The list of track labels
  QPtrList<QLabel> trackLabelList;
  /// Last number highlighted because of current step
  QLabel *lastNumberLabel;
  /// Size in steps
  unsigned int size;
  /// Increments to the size
  unsigned int step;
  /// Position in the view of the cursor
  unsigned int rpos, cpos;
  /// Whether it needs full data (pattern name at top...)
  bool showTitles;
  /// Loop region
  int loopBegin, loopEnd;
  /// Number where first clicked ocurred
  int loopFirstClick;

  QString textOf(int row, int trackN);
signals:
  /// Set the size (rows) of the trackview
  void sizeChanged(unsigned int size);
  /// Signal to request the active pattern
  void setActivePatternViewInSequence(int patternId);
  /// A new loop is selected
  void newLoop(int begin, int end);
  /// Sets the position at the status bar
  void positionChanged(unsigned char,unsigned char);
  
public slots:
  void manageTrack(Track *tr);
  void unmanageTrack(Track *tr); 
  void setSizeSlot(unsigned int size);
  void setPosition(unsigned int rpos, unsigned int cpos);
  void loopChanged(int begin, int end);

  void trackNameChanged(Track *,const QString &);

  void contentsMousePressEvent(QMouseEvent *e);
  void contentsMouseMoveEvent(QMouseEvent *e);
  void contentsMouseReleaseEvent(QMouseEvent *e);
  void keyPressEvent(QKeyEvent *);
  void focusInEvent(QFocusEvent *);
  void focusOutEvent(QFocusEvent *);

  void redrawPatterns();
  void redrawNumbers();
  void setPosition(int step);

  void muteChanged(Track *, bool );
  void setSignature(int a,int b);
};

#endif 
