/***************************************************************************
 *   Copyright (C) 2004,2005 by David Moreno Montero                       *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#include "sysex.h"
#include <kmessagebox.h>

/**
 * @short Calculates the SysEx length
 */
int SysEx::sysexlen(const char *msg){
  int i=1;
  while((msg[i]&0x0FF)!=0x0F7){
    i++;
  }
  return i+1;
}

/**
 * @short Checks a SysEx Message is valid or not
 */
/// FIXME make me complete
bool SysEx::validSysEx(const char *msg, int size){
  if ((msg[size-1]&0x0FF)!=0x0F7){
    printf("%02X\n",msg[size-1]);
    return false;
  }
  return true;
}


/**
 * @short Creates a SysEx Messahe with a name and message
 */
SysEx::SysEx(const QString &_name,char *_msg):QObject(NULL,_name){
  name=_name;
  setMessage(_msg);
}

/**
 * @short Deletes all unused data and tells everybody
 */
SysEx::~SysEx(){
  int i;
  
  // I delete params...
  for (i=0;i<numParams();i++)
    delParam(i);

  emit delSysEx(name);
  emit delSysEx(this);
} 

/**
 * Adds a parameter in the SysEx message
 */
int SysEx::addParam(int pos, int len,const QString &_name){
  params.append(pos);
  paramlen.append(len);
  paramName.append(_name);

  emit newParam(name,_name);
  return numParams()-1;
}

/**
 * Sets the value of a SysEx message
 */
void SysEx::setParam(int paramn, float value){
  if (paramn<0 || paramn>=numParams())
    return;

  int len=paramlen[paramn];
  int pos=params[paramn];
  switch (len){
  case 1:
    {
      unsigned int val;
      val=int(value*63.5)+64;
      msg[pos]=val;
    }
    break;
  case 2:
    {
      unsigned int val;
      val=int(value*8191.5)+8191.5;
      msg[pos]=(val&0x03F80)>>7;
      msg[pos+1]=(val&0x07F);
    }
    break;
  default:
    printf("SysEx controller size invalid (%d)\n",len);
  }
}

/**
 * @short Changes the message to _msg
 */
void SysEx::setMessage(const char *_msg){
  int len=sysexlen(_msg);
  msg=QByteArray(len);

  int i;
  for (i=0;i<len;i++)
    msg[i]=_msg[i];
}

/**
 * @short Sets the message from the QString. This message is in HEX coded in
 * ASCII.
 */
int SysEx::setMessage(const QString &text){
  QStringList l=QStringList::split(" ",text.stripWhiteSpace());
  QStringList::iterator si,siEnd;
  QString nH;
  unsigned int i;
  bool ok;
  QValueList<char> msg;

  for (si=l.begin(),siEnd=l.end();si!=siEnd;++si){
    nH=*si;
    msg.append(nH.toInt(&ok,16));
    if (!ok){
      KMessageBox::error(NULL,"Invalid input","Current SysEx Message is invalid.\n"
			 " It contains non-Hexadecimal values");
      return 0;
    }
  }
  
  char *cmsg=new char[msg.count()];
  for (i=0;i<msg.count();i++)
    cmsg[i]=msg[i];
  
  if (!validSysEx(cmsg,msg.count())){
    KMessageBox::error(NULL,"Not a valid SysEx Message",
		       "This is not a valid System Exclusive message.");
    delete cmsg;
    return 0;
  }
  setMessage(cmsg);
  delete[] cmsg;
  return 1;
}

/**
 * @short Removes a parameter
 */
void SysEx::delParam(int param){
  if (param>=(int)params.count())
    return;

  emit deletedParam(name, param);

  paramName.remove(paramName.at(param));
  params.remove(params.at(param));
  paramlen.remove(paramlen.at(param));
}
