/***************************************************************************
 *   Copyright (C) 2004,2005 by David Moreno Montero                       *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#ifndef __SYSEX_H__
#define __SYSEX_H__

#include <qobject.h>
#include <qvaluelist.h>
#include <qcstring.h>
#include <qstringlist.h>

/**
 * @short Contains a SysEx message
 *
 * The SysEx message is a byte stream that begins with 0xF0, ends with
 * 0x7F, and have, maybe, some variable parameters.
 */
class SysEx:public QObject{
  Q_OBJECT
 protected:
  /// The sysex msg itself
  QByteArray msg;
  /// Names of parameters
  QStringList paramName;
  /// Start point of params
  QValueList<int> params;
  /// Length of params
  QValueList<char> paramlen;
  /// Name of message
  QString name;

  static int sysexlen(const char *);
 public:
  static bool validSysEx(const char *msg, int size);
 signals:
  void newParam(const QString &sysexname, const QString &paramname);
  void deletedParam(const QString &sysexname, int paramn);
  void delSysEx(const QString &sysexname);
  void delSysEx(SysEx *);

 public:
  SysEx(const QString &name, char *msg=NULL);
  ~SysEx();

  int addParam(int pos, int len, const QString &name);
  void setParam(int paramn, float value);

  void delParam(int paramn);

  int numParams(){ return params.count(); };
  int paramPosition(int npar){ return params[npar]; };
  int paramLength(int npar){ return paramlen[npar]; };
  QString getName(int npar){ return paramName[npar]; };
  QString getName(){ return name; };
  void setName(const QString &_name){ name=_name; };
  void setMessage(const char *msg);
  int setMessage(const QString &msg);
  int length(){ return msg.count(); };

  /// Maybe needs .copy(). Investigate FIXME
  QByteArray getSysEx(){ return msg; };
};

#endif

