/***************************************************************************
 *   Copyright (C) 2004,2005 by David Moreno Montero                       *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/


#include "sysex.h"
#include "sysexbag.h"

SysExBag::SysExBag():QObject(NULL,"sysexbag"){
  sysex.setAutoDelete(true);
  lastSysExIncomplete=false;
}

void SysExBag::append(SysEx *s){
  sysex.append(s);

  printf("New sysex at bag\n");
  connect(s,SIGNAL(newParam(const QString &,const QString &)),
	  this, SIGNAL(newSysExParam(const QString &,const QString &)));
  connect(s,SIGNAL(deletedParam(const QString &,int)),
	  this, SIGNAL(delSysExParam(const QString &,int)));
  connect(s,SIGNAL(delSysEx(const QString &)),
	  this, SIGNAL(delSysEx(const QString &)));
  emit newSysEx(s->getName());
}

SysEx *SysExBag::byName(const QString &name){
  unsigned int i;
  for (i=0;i<count();i++)
    if (at(i)->getName()==name)
      return at(i);
  return NULL;
}

void SysExBag::remove(const QString &name){
  SysEx *s=byName(name);

  sysex.remove(s);

  printf("Deleting sysex message %s\n",name.ascii());
  emit delSysEx(name);
}

void SysExBag::clear(){
  int i;

  printf("Clearing SysEx Bag\n");

  while(count()>0)
    remove(at(0)->getName());

}

/**
 * @short Appends an unnamed sysex to unnamed sysex list
 */
void SysExBag::appendUnnamed(const char *data, int len){
  QByteArray a;
  if (lastSysExIncomplete==false){
    printf("start %d\n",len);
    a.duplicate(data,len);
    
    unnamed.push_front(a);
    
    // Keep a limit in size
    if (unnamed.count()>20)
      unnamed.pop_back();
  }
  // last was incomplete, so this is appended
  else{
    a=unnamed[0];
    printf("completing + %d (%d)\n",len,a.size()+len);
    char *d=new char[a.size()+len];
    memcpy(d,a.data(),a.size());
    memcpy(&d[a.size()], data, len);
    a.duplicate(d,a.size()+len);
    unnamed[0]=a;
  }

  if ((data[len-1]&0x0FF)!=0x0F7)
    lastSysExIncomplete=true;
  else
    lastSysExIncomplete=false;

}
