/***************************************************************************
 *   Copyright (C) 2004,2005 by David Moreno Montero                       *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#ifndef __SYSEX_BAG__
#define __SYSEX_BAG__

#include <qobject.h>
#include <qptrlist.h>
#include <qvaluelist.h>
#include <qstring.h>
#include <qmemarray.h>

class SysEx;

class SysExBag:public QObject{
  Q_OBJECT
protected:
  /// List of known sysex messages
  QPtrList<SysEx> sysex;
  /// List of sysexs received by midi data.
  QValueList<QByteArray> unnamed;
  /// Last sysex message captured was incomplete
  bool lastSysExIncomplete;
public:
  SysExBag();

  /// @{ @name Normal Bag
  SysEx *at(unsigned int pos){ return sysex.at(pos); };
  unsigned int count(){ return sysex.count(); }
  SysEx *byName(const QString &name);
  /// @}

  /// @{ @name Unname Bag
  QByteArray unnamedAt(unsigned int pos){ return unnamed[pos]; }
  unsigned int unnamedCount(){ return unnamed.count(); }
  /// @}

  void clear();

public slots:
  void append(SysEx *s);
  void remove(const QString &); 

  void appendUnnamed(const char *data, int len);
  
signals:
  void newSysEx(const QString &name);
  void newSysExParam(const QString &sysexname,const QString &paramname);
  void delSysExParam(const QString &sysexname, int paramn);
  void delSysEx(const QString &sysexname);
};


#endif
