/***************************************************************************
 *   Copyright (C) 2004,2005 by David Moreno Montero                       *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#include "sysexeditor.h"
#include "sysex.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qcombobox.h>
#include <qtextedit.h>
#include <qmessagebox.h>
#include <qfile.h>

#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kinputdialog.h>
#include <klistbox.h>
#include <klocale.h>


/**
 * @short Creates the SysEx Editor
 */
SysExEditor::SysExEditor(SysExBag *sysexbag, QWidget *parent):
  KDialog(parent,"sysex_editor"){
  sysex=sysexbag;
  actualSysEx=NULL;
  setCaption("SysEx Editor");

  setSizeGripEnabled(TRUE);

  QVBoxLayout *layout=new QVBoxLayout(this,11,6,"layout");
  QGroupBox *group;
  QHBox *hbox;
  QVBox *vbox;
  QLabel *label;
  QPushButton *button;

  group=new QGroupBox("SysEx",this);
  group->setColumns(1);
  vbox=new QVBox(group);
  vbox->setSpacing(6);
  hbox=new QHBox(vbox); 
  hbox->setSpacing(6);
  label=new QLabel("Name: ",hbox);
  nameCombo=new QComboBox(hbox,"sysex_name_combo");
  nameCombo->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::Fixed);
  connect(nameCombo,SIGNAL(highlighted(const QString &)), this, 
	  SLOT(setActualSysEx(const QString &)));
  hbox=new QHBox(vbox);
  hbox->setSpacing(6);
  button=new QPushButton("New",hbox);
  connect(button,SIGNAL(clicked()),this,SLOT(newSysEx()));
  editName=new QHBox(hbox);
  button=new QPushButton("Rename",editName);
  button=new QPushButton("Remove",editName);
  connect(button,SIGNAL(clicked()),this,SLOT(delSysEx()));
  group->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::Fixed);
  layout->addWidget(group);

  edit=new QVBox(this);
  group=new QGroupBox("Message",edit);
  group->setColumns(1);
  vbox=new QVBox(group);
  vbox->setSpacing(6);
  message=new QTextEdit(vbox,"sysex_message");
  message->setReadOnly(true);
  group->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::MinimumExpanding);
  hbox=new QHBox(vbox);
  hbox->setSpacing(6);
  button=new QPushButton("Edit Message",hbox);
  connect(button,SIGNAL(clicked()), this, SLOT(enterNewMessage()));
  button=new QPushButton("Load from File",hbox);
  connect(button,SIGNAL(clicked()), this, SLOT(loadNewMessage()));
  button=new QPushButton("Load from Received",hbox);
  connect(button,SIGNAL(clicked()), this, SLOT(loadNewMessageFromUnnamed()));

  group=new QGroupBox("Controller",edit);
  group->setColumns(1);
  vbox=new QVBox(group);
  vbox->setSpacing(6);
  controllerCombo=new QComboBox(vbox,"sysex_controller_combo");
  connect(controllerCombo,SIGNAL(highlighted(int)),this,SLOT(selectedController(int)));
  hbox=new QHBox(vbox);
  hbox->setSpacing(6);
  button=new QPushButton("New on Selected",hbox);
  connect(button,SIGNAL(clicked()),this,SLOT(newController()));
  button=new QPushButton("Rename",hbox);
  button=new QPushButton("Remove",hbox);
  connect(button,SIGNAL(clicked()),this,SLOT(delController()));
  layout->addWidget(edit);

  button=new QPushButton("Close",this);
  connect(button,SIGNAL(clicked()), this, SLOT(close()));
  layout->addWidget(button);


  connect(sysex, SIGNAL(newSysEx(const QString &)),
	  this, SLOT(addSysEx(const QString &)));
  connect(sysex, SIGNAL(delSysExParam(const QString &, int)),
	  this, SLOT(delController(const QString &,int)));
  connect(sysex, SIGNAL(delSysEx(const QString &)),
	  this, SLOT(deletedSysEx(const QString &)));


  edit->setEnabled(false);
  editName->setEnabled(false);
}

/**
 * @short Adds a new sysEx. Asks for the name
 */
void SysExEditor::newSysEx(){
  QString name;
  bool ok;

  name=KInputDialog::getText("SysEx Name","SysEx Message Name","",&ok,this);
  if (ok){
    newSysEx(name);
  }
}

/**
 * @short Adds a new controller (sysex parameter). Asks for the name.
 */
void SysExEditor::newController(){
  QString name;
  bool ok;

  if (message->hasSelectedText()==false)
    return;

  name=KInputDialog::getText("SysEx Controller Name","SysEx Controller Name","",&ok,this);
  if (ok){
    newController(name);
  }
}

/**
 * @short Deletes the selected SysEx
 */
void SysExEditor::delSysEx(){
  int ok;
  ok=KMessageBox::questionYesNo(this,
				QString("Are you sure you want to delete"
					" SysEx Message %1?\n"
					"All related controllers will be deleted too...").
				arg(nameCombo->currentText()),
				"Sure to delete?");

  if (ok==KMessageBox::Yes){
    sysex->remove(nameCombo->currentText());
  }
}

/**
 * @short A SysEx was deleted
 */
void SysExEditor::deletedSysEx(const QString &name){
  int i;
  for (i=0;i<nameCombo->count();i++)
    if (nameCombo->text(i)==name){
      nameCombo->removeItem(i);
      setActualSysEx(nameCombo->currentText());
    }

}

/**
 * @short Delete a controller
 */
void SysExEditor::delController(){
  int ok;
  ok=KMessageBox::questionYesNo(this,
				QString("Are you sure you want to delete"
					" SysEx Controller\n%1").
				arg(controllerCombo->currentText()),
				"Sure to delete?");

  if (ok==KMessageBox::Yes){
    actualSysEx->delParam(controllerCombo->currentItem());
  }
}

/**
 * @short Adds a SysEx message with the given name
 */
void SysExEditor::newSysEx(const char *name){
  char emptySysEx[]={0xF0, 0xF7};

  SysEx *s=new SysEx(name,emptySysEx);
  sysex->append(s);
  actualSysEx=s;
  setActualSysEx(name);
}

/**
 * @short A sysexMessage was added, I add it to my lists
 */
void SysExEditor::addSysEx(const QString &name){
  nameCombo->insertItem(name);
  nameCombo->setCurrentItem(nameCombo->count()-1);
  sysExNames+=name;
  if (actualSysEx==NULL){
    setActualSysEx(name);
  }
}


/**
 * @short Create new controller on selection with given name
 */
void SysExEditor::newController(const char *name){

  if (message->hasSelectedText()==false)
    return;

  int ps, pi, cs, ci;
  message->getSelection(&ps,&cs,&pi,&ci);

  int s,l;
  s=(cs+1)/3;
  l=(ci-cs)/3+1;

  if (s==0){
      KMessageBox::error(this,"Invalid selection",
			 "Current selection is invalid as it\n"
			 "contains the first (F0) character");
      return;
  }
  if ((s+l)>=actualSysEx->length()){
      KMessageBox::error(this,"Invalid selection",
			 "Current selection is invalid as it\n"
			 "contains the last (F7) character");
      return;
  }

  if (NULL==actualSysEx)
    return;

  controllerCombo->insertItem(name);
  controllerCombo->setCurrentItem(controllerCombo->count()-1);

  actualSysEx->addParam(s,l, name);
  selectedController(controllerCombo->count()-1);
}

/**
 * @short Deletes the given controller 
 *
 * Its from name SysEx message, parameter param
 */
void SysExEditor::delController(const QString &name, int param){
  if (name==actualSysEx->getName()){
    controllerCombo->removeItem(param);
  }
}

/**
 * @short Changes the actual SysEx message to given one
 */
void SysExEditor::setActualSysEx(const QString &name){
  actualSysEx=sysex->byName(name);
  if (NULL==actualSysEx){
    setSysExMessage(NULL,0);
    edit->setEnabled(false);
    editName->setEnabled(false);
  }
  else{
    edit->setEnabled(true);
    editName->setEnabled(true);
    setSysExMessage(actualSysEx->getSysEx().data(),actualSysEx->getSysEx().count());
    controllerCombo->clear();
    int i;
    for (i=0;i<actualSysEx->numParams();i++)
      controllerCombo->insertItem(actualSysEx->getName(i));
    selectedController(controllerCombo->currentItem());
  }
}

/**
 * @short Changes the SysEx message to given data
 */
void SysExEditor::setSysExMessage(const char *data, int count){
  QString msg;
  int i;

  for (i=0;i<count;i++)
    msg+=QString("00%1").arg(int(data[i]),0,16).right(2).upper()+" ";
  
  message->setText(msg);
}

/**
 * @short Sets the sysex message from data in HEX coded in ASCII
 */
void SysExEditor::setSysExMessageHex(const QString &text){
  if (NULL!=actualSysEx){
    if (!actualSysEx->setMessage(text))
      enterNewMessage(text);
    else{
      QByteArray msg=actualSysEx->getSysEx();
      setSysExMessage(msg.data(),msg.count());
    }
    return;
  }
}

/**
 * @short Asks for a new message text from current text
 */
void SysExEditor::enterNewMessage(){
  enterNewMessage(message->text());
}

/**
 * @short Asks for a new message from given data
 */
void SysExEditor::enterNewMessage(const QString &initial){
  bool ok;
  QString text;
  text=KInputDialog::getMultiLineText("Enter SysEx Message",
				      "Enter SysEx Message",initial,&ok,this);
  if (!ok)
    return;

  setSysExMessageHex(text);
}

/**
 * @short Loads the message from binary file
 */
void SysExEditor::loadNewMessage(){
  QString file;
  file=KFileDialog::getOpenFileName("::sysex_messages", 
				    QString::null, this, 
				    "Select SysEx Hex Message to Open");
  if (file==QString::null)
    return;
  QFile f(file);
  if (f.open(IO_ReadOnly)==false){
    KMessageBox::error(this,"Can't open selected file","Error opening file");
    return;
  }
  
  QByteArray msg;
  msg=f.readAll();
  if (!SysEx::validSysEx(msg.data(),msg.count())){
    KMessageBox::error(this,"Not a valid SysEx Message",
		       "This is not a valid System Exclusive message.");
    return;
  }
  
  setSysExMessage(msg.data(),msg.count());
  if (NULL!=actualSysEx)
    actualSysEx->setMessage(msg.data());

}

/**
 * @short Shows a list of unnamed, and chooses one from it
 */
void SysExEditor::loadNewMessageFromUnnamed(){
  int i,j;
  QStringList sysex_list;

  for (i=0;i<sysex->unnamedCount();i++){
    QString msg;
    QByteArray ba=sysex->unnamedAt(i);
    for (j=0;j<ba.count();j++)
      msg+=QString("00%1").arg(int(ba[j]),0,16).right(2).upper()+" ";
    sysex_list.append(msg);
  }

  KDialogBase *dialog=new KDialogBase(this, i18n("Choose SysEx Message"),
				      true, i18n("Choose SysEx Message"),
				      KDialogBase::Ok|KDialogBase::Cancel,
				      KDialogBase::Ok, true);

  KListBox *lb=new KListBox(dialog);
  lb->insertStringList(sysex_list);

  dialog->setMainWidget(lb);
  dialog->show();
  int ret=dialog->exec();

  if (ret==KDialogBase::Cancel)
    return;

  QByteArray msg=sysex->unnamedAt(lb->currentItem());
  setSysExMessage(msg.data(),msg.count());
  if (NULL!=actualSysEx)
    actualSysEx->setMessage(msg.data());
}


/**
 * @short Selected controller changed, updates data.
 */
void SysExEditor::selectedController(int n){
  int s,l;

  if (NULL==actualSysEx)
    return;
  if (actualSysEx->numParams()>=n)
    return;
  s=actualSysEx->paramPosition(n)*3;
  l=actualSysEx->paramLength(n)*3;

  message->setSelection(0,s-1,0,s+l);
}
