/***************************************************************************
 *   Copyright (C) 2004 by David Moreno Montero                            *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

#ifndef __TRACKERBOX_H__
#define __TRACKERBOX_H__

#include <qwidget.h>

class Tracker;
class KMainWindow;
class Track;
class KDoubleSpinBox;
class MidiNote;
class QLabel;
class Midi;
class KAction;
class KToggleAction;
class SignatureWidget;
class SysExBag;

/**
 * @short Tracker controller
 *
 * This class creates a widget that can control the tracker flow. It
 * can play, stop, pause, change BPM...
 *
 * This class is more about a container of related KAction s to
 * Tracker.
 */
class TrackerBox:public QWidget{
  Q_OBJECT
public:
  TrackerBox(SysExBag *bag, KMainWindow *parent, const char *name=0);
  ~TrackerBox();

  float getBPM();
  int getLoopBegin();
  int getLoopEnd();
  bool isLoopActive();
  Midi *getMidi();
  Tracker *getTracker();

signals:
  /// Playing should start
  void startPlay();
  /// Playign should stop
  void stopPlay();
  /// Activates the loop region
  void activateLoop(bool);

public slots:
  void addTrack(Track *);
  void delTrack(Track *);
  void stepSlot(int step);

  void playStop();

  void newBPM(double);

  void setPlaying();
  void setNotPlaying();

  void loopActivated(bool loop);

protected:
  /// The real tracker. Is another thread.
  Tracker *tracker;
  /// The BPM spin box
  KDoubleSpinBox *sp;
  /// Actual time
  QLabel *time;
  /// Actions, should be acesible as they can be activated or deactivated
  KAction *play,*stop, *playstop;
  /// Whether or not activate the loop
  KToggleAction *playLoop;
  /// Combo with all combinations
  SignatureWidget *signatures;
};

#endif
