/***************************************************************************
 *   Copyright (C) 2004 by David Moreno Montero                            *
 *   david@monasteriomono.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/


#ifndef __TRACK_VIEW_H__
#define __TRACK_VIEW_H__

#include <qvbox.h>
#include <qstring.h>

#include "track.h"

class QSplitter;
class PatternView;
class KListBox;
class QWidgetStack;
class SequenceView;
class QLineEdit;
class ALSAPort;
class QComboBox;
class QSpinBox;
class QListBox;
class QToolBox;

/**
 * @short The view of the Track class.
 *
 * The view of the track. This version have three panels. In one you
 * have a PatternView, in the seccond you have a selectable list of
 * patterns (with a Id attached), and in the third you have a
 * SequenceView.
 *
 * This class is just the View (in MVC) of the Track.
 */
class TrackView:public QVBox{
  Q_OBJECT
public:
  TrackView(Track *tr, QWidget *parent, const char *name);
  ~TrackView();
  
  Track *getTrack(){ return track; };

private:
  void constructView();

protected:
  /// Viewed class
  Track *track;
  /// Horizontal division of the view
  QSplitter *view;
  /// Tool box that contains all
  QToolBox *tbox;
  /// A Stack of views of the patterns
  QWidgetStack *patternView;
  /// The selectable list of patterns
  KListBox *patternList;
  /// The sequence editor
  SequenceView *sequenceView;
  /// Current view of the playing pattern. Used to show the step pointer
  PatternView *currentPlayingView;
  /// The editor of the pattern name
  QLineEdit *patternNameEdit;
  /// Combo to select ALSA output
  QComboBox *alsaCombo;
  /// Spin to select base octave
  QSpinBox *baseOctave;
  /// List of initial SysEx
  QListBox *initialSysEx;
  /// Rows of current pattern spin
  QSpinBox *rowsSpin;
  /// Columns of current pattern 
  QSpinBox *columnsSpin;
  /// Currently in follow song mode
  bool followSong;
  
signals:
  /// A pattern changed some important data
  void patternChanged();
  /// Activate Track KAction s
  void showTrackMenu();
  /// Shows sysex editor
  void showSysExEditor();
  /// Adds the initial sysex command
  void addInitialSysEx(SysEx *);
  /// Adds the initial sysex command
  void delInitialSysEx(SysEx *);
  /// Shows a status message at status bar
  void statusMessage(const QString&,int);
  /// Changes the position at the status bar
  void positionChanged(unsigned char cpos, unsigned char rpos);
  /// Select new loop
  void newLoop(int begin, int end);
  /// The loop region have changed
  void loopChanged(int begin, int end);
  /// Select the ALSA output
  void setALSAOutput(Track *tr,int client, int port);
  /// Select the ALSA output by name
  void setALSAOutput(const QString &name);
  /// Emmited when config settings changes
  void reloadSettingsSignal();

public slots:
  void setPatternName(int id, const QString &newName);
  void addPattern(void);
  void showPatternProperties(void);
  void showControllersProperties(void);
  void copyPattern(void);
  void newPatternAdded(Pattern *pattern);
  void aboutToShowPatternView(QWidget*);
  void redrawStepPointer(int step);
  void selectedPattern(int id);
  void changeCurrentPatternName(const QString &nname);
  void changeRowsAndColumns();
  void deletedPattern(Pattern *pt);
  void reloadSettings();
  
  void addInitialSysEx();
  void addedInitialSysEx(SysEx *);
  void delSelectedInitialSysEx();
  void deletedInitialSysEx(SysEx *);
  
  void addColumn();
  void delColumn();
  void addController();
  void delController();

  void delSelectedPattern();

  void trackDeleted(Track *tr);

  void setActivePatternView();
  void setActivePatternList();
  void setActiveSequenceView();

  void midiNote(int step,const MidiNote &ev);
  void ALSAPortList(QPtrList<ALSAPort> &l);
  void alsaComboSelected(const QString &);
  
  void doCopy();
  void doPaste();
};



#endif
